% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% generate data for: channel - reader/tag antenna directivity
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
%
% *******************************************************************************************************


% ******************************************************************************************************
% initialization
clear; close all; clc;
version = 'beta 1.0';

% one last chance to reconsider
disp(sprintf('WARNING: This will overwrite existing files!\n'));
reply = input('Are you REALLY sure y/n [n]?  ', 's');
if ~strcmpi(reply, 'y')
   return
end
disp(' ');

% paths, etc
%     path to globalinit.m
path = '/afs/spsc.tugraz.at/user/arnitz/spr/matlab/';
%     Matlab does not resolve symbolic links; if we're running on Unix, let the system resolve them
if isunix
   [dummy, path] = system(sprintf('readlink -f "%s"', path));
end
%     add this path and call globalinit script
addpath(path); clear('dummy', 'path');
globalinit('init');

% "switch to here"
cd(fileparts(mfilename('fullpath')));



% ******************************************************************************************************
% settings / generate base vectors (angles for azimuth and elevation)

% settings
settings.res        =       1; % number of points per degree
settings.plotfolder = 'plots'; % subfolder for plots

% base vectors for azimuth and elevation
% ... include 0 and 360 degree to simplify interpolation
az = 0 : 1/settings.res : 360;
el = 0 : 1/settings.res : 180;



% ******************************************************************************************************
% intermec reader antenna IA39b

% load data
usedmatfiles = 'meas_antenna_intermec-ia39b';
data         = load(usedmatfiles);

% interpolate to new angles
az_gain = interp1(data.az, data.az_gain, az, 'spline', 'extrap');
el_gain = interp1(data.el, data.el_gain, el, 'spline', 'extrap');

% prepare everything and save
%     header
matfilename    = 'channelchar_directivity_intermec-ia39b'; %#ok<NASGU>
characteristic = 'radiation pattern for intermec ia39b reader antenna: gain(az,el) = az_gain(az) * el_gain(el) (2x2D)'; %#ok<NASGU>
createdon      = datestr(now, 0); %#ok<NASGU>
createdby      = sprintf('%s.m, rev.: %s', mfilename, version); %#ok<NASGU>
usedmatfiles   = ''; %#ok<NASGU>
%     data
az = az(:); el = el(:);
az_gain(isnan(az_gain)) = 0; az_gain = az_gain(:);
el_gain(isnan(el_gain)) = 0; el_gain = el_gain(:);
full3d  = false; % true: gain(az,el);   false: az_gain(az), el_gain(el)
% save
save(fullfile('channel',matfilename), 'matfilename','characteristic','createdby','usedmatfiles',...
   'az','az_gain', 'el','el_gain', 'full3d');

% 2D plots (first hold off to draw grid)
figure;
hold off; polar(el *pi/180, el_gain, 'b.');
hold on;  polar(az *pi/180, az_gain, 'r.');
polar(data.el *pi/180, data.el_gain, 'b-');
polar(data.az *pi/180, data.az_gain, 'r-');
hold off; setlegend({'elevation','azimuth'}, 'NorthEastOutside'); 
setlabels('ANTENNA RADIATION PATTERN (DIRECTIVITY): INTERMEC IA39b', '', '');
savefigure(gcf, fullfile(settings.plotfolder, sprintf('%s-2d',matfilename)), 'png');

% 3D plot (just for the view)
%     transform from sphere to cartesian coordinates ... gain(az,el) = gain(az) * gain(el)
x = (el_gain .* sin(el*pi/180)) * (az_gain .* cos(az*pi/180))';
y = (el_gain .* sin(el*pi/180)) * (az_gain .* sin(az*pi/180))';
z = (el_gain .* cos(el*pi/180)) * (az_gain                  )';
rho = sqrt( x.^2 + y.^2 + z.^2 );
%     plot
figure; hold on;
surface(x, y, z, rho, 'EdgeColor','none', 'FaceColor','interp');
plot3( az_gain .* cos(az*pi/180), az_gain .* sin(az*pi/180), zeros(size(az)), 'k-'); % horz pattern
plot3( el_gain .* sin(el*pi/180), zeros(size(el)), el_gain .* cos(el*pi/180), 'k-'); % vert pattern
% plot3( cos(az*pi/180), sin(az*pi/180), zeros(size(az)), 'k--'); % unit circe xy
% plot3( sin(el*pi/180), zeros(size(el)), cos(el*pi/180), 'k--'); % unit circle yz
% plot3(-sin(el*pi/180), zeros(size(el)), cos(el*pi/180), 'k--');
hold off; grid on; view([30, 60]); setlegend({'3D=2x2D', 'horizontal, vertical'}, 'NorthWest');
setlabels('ANTENNA RADIATION PATTERN (DIRECTIVITY): INTERMEC IA39b', 'x', 'y', 'z');
savefigure(gcf, fullfile(settings.plotfolder, sprintf('%s-3d',matfilename)), 'png');



% ******************************************************************************************************
% ideal lambda/4 dipole 

% dipole antenna pattern (dipole in z-axis)
az_gain = ones(size(az));
el_gain = ( cos(pi/2 * cos(el*pi/180)) - cos(pi/2) ) ./ sqrt( 1 - cos(el*pi/180).^2 );

% prepare everything and save
%     header
matfilename    = 'channelchar_directivity_l4-dipole'; %#ok<NASGU>
characteristic = 'radiation pattern for lambda/4 dipole: gain(az,el) = az_gain(az) * el_gain(el) (2x2D)'; %#ok<NASGU>
createdon      = datestr(now, 0); %#ok<NASGU>
createdby      = sprintf('%s.m, rev.: %s', mfilename, version); %#ok<NASGU>
usedmatfiles   = ''; %#ok<NASGU>
%     data
az = az(:); el = el(:);
az_gain(isnan(az_gain)) = 0; az_gain = az_gain(:);
el_gain(isnan(el_gain)) = 0; el_gain = el_gain(:);
full3d  = false; % true: gain(az,el);   false: az_gain(az), el_gain(el)
% save
save(fullfile('channel',matfilename), 'matfilename','characteristic','createdby','usedmatfiles',...
   'az','az_gain', 'el','el_gain', 'full3d');

% 2D plots (first hold off to draw grid)
figure;
hold off; polar(el *pi/180, el_gain, 'b');
hold on;  polar(az *pi/180, az_gain, 'r');
hold off; setlegend({'elevation','azimuth'}, 'NorthEastOutside'); 
setlabels('ANTENNA RADIATION PATTERN (DIRECTIVITY): \lambda/4 DIPOLE', '', '');
savefigure(gcf, fullfile(settings.plotfolder, sprintf('%s-2d',matfilename)), 'png');

% 3D plot (just for the view)
%     transform from sphere to cartesian coordinates ... gain(az,el) = gain(az) * gain(el)
x = (el_gain .* sin(el*pi/180)) * (az_gain .* cos(az*pi/180))';
y = (el_gain .* sin(el*pi/180)) * (az_gain .* sin(az*pi/180))';
z = (el_gain .* cos(el*pi/180)) * (az_gain                  )';
rho = sqrt( x.^2 + y.^2 + z.^2 );
%     plot
figure; hold on;
surface(x, y, z, rho, 'EdgeColor','none', 'FaceColor','interp');
plot3( az_gain .* cos(az*pi/180), az_gain .* sin(az*pi/180), zeros(size(az)), 'k-'); % horz pattern
plot3( el_gain .* sin(el*pi/180), zeros(size(el)), el_gain .* cos(el*pi/180), 'k-'); % vert pattern
% plot3( cos(az*pi/180), sin(az*pi/180), zeros(size(az)), 'k--'); % unit circe xy
% plot3( sin(el*pi/180), zeros(size(el)), cos(el*pi/180), 'k--'); % unit circle yz
% plot3(-sin(el*pi/180), zeros(size(el)), cos(el*pi/180), 'k--');
hold off; grid on; view([30, 60]); setlegend({'3D=2x2D', 'horizontal, vertical'}, 'NorthWest');
setlabels('ANTENNA RADIATION PATTERN (DIRECTIVITY): \lambda/4 DIPLOLE', 'x', 'y', 'z');
savefigure(gcf, fullfile(settings.plotfolder, sprintf('%s-3d',matfilename)), 'png');



% ******************************************************************************************************
% ideal lambda/2 dipole 

% dipole antenna pattern (dipole in z-axis)
az_gain = ones(size(az));
el_gain = ( cos(pi * cos(el*pi/180)) - cos(pi) ) ./ sqrt( 1 - cos(el*pi/180).^2 );

% prepare everything and save
%     header
matfilename    = 'channelchar_directivity_l2-dipole'; %#ok<NASGU>
characteristic = 'radiation pattern for lambda/2 dipole: gain(az,el) = az_gain(az) * el_gain(el) (2x2D)'; %#ok<NASGU>
createdon      = datestr(now, 0); %#ok<NASGU>
createdby      = sprintf('%s.m, rev.: %s', mfilename, version); %#ok<NASGU>
usedmatfiles   = ''; %#ok<NASGU>
%     data
az = az(:); el = el(:);
az_gain(isnan(az_gain)) = 0; az_gain = az_gain(:);
el_gain(isnan(el_gain)) = 0; el_gain = el_gain(:);
full3d  = false; % true: gain(az,el);   false: az_gain(az), el_gain(el)
% save
save(fullfile('channel',matfilename), 'matfilename','characteristic','createdby','usedmatfiles',...
   'az','az_gain', 'el','el_gain', 'full3d');

% 2D plots (first hold off to draw grid)
figure;
hold off; polar(el *pi/180, el_gain, 'b');
hold on;  polar(az *pi/180, az_gain, 'r');
hold off; setlegend({'elevation','azimuth'}, 'NorthEastOutside'); 
setlabels('ANTENNA RADIATION PATTERN (DIRECTIVITY): \lambda/2 DIPOLE', '', '');
savefigure(gcf, fullfile(settings.plotfolder, matfilename), 'png');

% 3D plot (just for the view)
%     transform from sphere to cartesian coordinates ... gain(az,el) = gain(az) * gain(el)
x = (el_gain .* sin(el*pi/180)) * (az_gain .* cos(az*pi/180))';
y = (el_gain .* sin(el*pi/180)) * (az_gain .* sin(az*pi/180))';
z = (el_gain .* cos(el*pi/180)) * (az_gain                  )';
rho = sqrt( x.^2 + y.^2 + z.^2 );
%     plot
figure; hold on;
surface(x, y, z, rho, 'EdgeColor','none', 'FaceColor','interp');
plot3( az_gain .* cos(az*pi/180), az_gain .* sin(az*pi/180), zeros(size(az)), 'k-'); % horz pattern
plot3( el_gain .* sin(el*pi/180), zeros(size(el)), el_gain .* cos(el*pi/180), 'k-'); % vert pattern
% plot3( cos(az*pi/180), sin(az*pi/180), zeros(size(az)), 'k--'); % unit circe xy
% plot3( sin(el*pi/180), zeros(size(el)), cos(el*pi/180), 'k--'); % unit circle yz
% plot3(-sin(el*pi/180), zeros(size(el)), cos(el*pi/180), 'k--');
hold off; grid on; view([30, 60]); setlegend({'3D=2x2D', 'horizontal, vertical'}, 'NorthWest');
setlabels('ANTENNA RADIATION PATTERN (DIRECTIVITY): \lambda/2 DIPLOLE', 'x', 'y', 'z');
savefigure(gcf, fullfile(settings.plotfolder, sprintf('%s-3d',matfilename)), 'png');
