% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% generate plots for: tag - modulation "light" characteristic (reflection coefficient)
%
% WARNING: Settings are not checked for sanity!
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
%
% *******************************************************************************************************

% *******************************************************************************************************
% initialization
clear; close all; clc; pause(0.01);
settings.version = 'beta 1.0';

% paths, etc
%     path to globalinit.m
path = fullfile(fileparts(mfilename('fullpath')), '..');
%     Matlab does not resolve symbolic links; if we're running on Unix, let the system resolve them
if isunix
   [dummy, path] = system(sprintf('readlink -f "%s"', path));
end
%     add this path and call globalinit script
restoredefaultpath; addpath(path); clear('dummy', 'path');
globalinit('silent');

% "switch to here"
cd(fileparts(mfilename('fullpath')));


% *******************************************************************************************************
% settings

%     general / file settings
settings.plotfolder = 'plots'; % subfolder for plots
settings.charid     = '_c-act'; % start with '_' if not empty
%     list of assembly combinations to plot (combination cat(1),rat(1) ... cat(2),rat(2) ... plotted)
%
settings.char_ind.cat  = [50:25:800, 850:50:1000, 1250, 1500, 2000:1000:5000]; % fF Cat
settings.char_ind.rat  = 0 * ones(size(settings.char_ind.cat)); % percent Rat shift
% 
% settings.char_ind.cat  = 2000 * ones(size(settings.char_ind.rat)); % fF Cat
% settings.char_ind.rat  = [-98, -95, -90, -80, -67, -50, -33, 0, 50, 100, 200, 500, 1000, 2000, 5000]; % percent Rat shift
%

% MFCW distance error estimates
settings.mfcw.df12 = 1e6; % Hz frequency differences between 2FCW carrier pair

% pav min/max and tick
settings.pav_min = 1e-5; % W
settings.pav_max =    3; % W
settings.pav_tick  = [1e-8, 1e-7, 1e-6, 1e-5, 1e-4, 1e-3, 1e-2, 1e-1, 1e0, 1e1]; % W

% order of unwrapping for phase, leave empty to deactivate unwrapping
settings.unwrap_rho = [2,1,3]; % reflection coefficient (1: f, 2: mod, 3: power); default: [2,1,3]
settings.unwrap_lin =   [1,2]; % linear model (1: f, 2: power); default: [1,2]

% clim for some plots
%     have to be set
settings.clim.rho_c_mag = [  0,    1]; % rho_c vs f and pav (linear model)
settings.clim.rho_d_mag = [  0, 0.40]; % rho_d vs f and pav (linear model)
settings.clim.rho_c_arg = [-pi,   2*pi]; % rad rho_c vs f and pav (linear model)
settings.clim.rho_d_arg = [-pi, 1.5*pi]; % rad rho_d vs f and pav (linear model)
%     can be empty ([] for auto)
settings.clim.mfcw_derr1     = [-2.0, 2.0]; % m 2FCW distance error ("full span" plot)
settings.clim.mfcw_derr2     = [-0.4, 0.4]; % m 2FCW distance error (zoomed plot)
settings.clim.darg_rho       = [ -10,  10]; % deg zoomed phase difference: rho_d(mod) - rho_d(unmod)

% figure options
settings.figopt.visible         =    'off'; % figure visibility {'on', 'off'} 
settings.figopt.type            =    'png'; % type of figures to create ('png' or 'eps'; leave empty to create both; see SAVEFIGURE)
settings.figopt.nonconfidential =    false; % remove confidential information (e.g. for presentations)?
settings.figopt.resolution      =      200; % dpi image resolution


% *******************************************************************************************************
% load light characteristic and process data

% output own name/version first
disp(sprintf('*******************************************************************************************************'));
disp(sprintf('* This is %s, version %s', mfilename, settings.version));
disp(sprintf('*******************************************************************************************************'));

% load characteristic (this might take some time)
disp(sprintf('\n Loading data (this might take a while)...'));
load(sprintf('tagchar_modulator%s-light', settings.charid));

% loop for set characteristics
for loop_ind = 1 : min(length(settings.char_ind.cat), length(settings.char_ind.rat))
   
   % find the appropriate index (not pretty, not at all)
   for i = 1 : length(v1_full)
      if v1_full(i) == settings.char_ind.cat(loop_ind) && v2_full(i) == settings.char_ind.rat(loop_ind)
         char_ind = i; break;
      end
   end
   % extract data and output what is done
   fch = tagchar_mod{char_ind}.fch;
   pav = tagchar_mod{char_ind}.pav;
   pic = tagchar_mod{char_ind}.pic;
   disp(sprintf('\n\n*******************************************************************************************************'));
   disp(sprintf('* WORKING ON: %s.mat [LIGHT] (created by %s)', tagchar_mod{char_ind}.matfilename, tagchar_mod{char_ind}.createdby));
   
   
   % ************************************************************
   % prepare data: linear model (rho_c + rho_{delta})
   disp(sprintf('   Preparing data for linear model...'));
   
   % decompose to linear model
   rho_c = ( squeeze(tagchar_mod{char_ind}.rho_pav(:, 1, :)) + squeeze(tagchar_mod{char_ind}.rho_pav(:, end, :)) ) / 2;
   rho_d =   squeeze(tagchar_mod{char_ind}.rho_pav(:, 1, :)) - rho_c;
   
   % magnitue
   mag.rho     = abs(tagchar_mod{char_ind}.rho_pav);
   mag.rho_c   = abs(rho_c);
   mag.rho_d   = abs(rho_d);
   
   % phase
   arg.rho     = angle(tagchar_mod{char_ind}.rho_pav);
   arg.rho_c   = angle(rho_c);
   arg.rho_d   = angle(rho_d);
   %     unwrap
   for i = settings.unwrap_rho
      arg.rho    (end:-1:1,:,end:-1:1) = unwrap(arg.rho    (end:-1:1,:,end:-1:1), [], i);
   end
   for i = settings.unwrap_lin
      arg.rho_c  (end:-1:1,  end:-1:1) = unwrap(arg.rho_c  (end:-1:1,  end:-1:1), [], i);
      arg.rho_d  (end:-1:1,  end:-1:1) = unwrap(arg.rho_d  (end:-1:1,  end:-1:1), [], i);
   end
   
   
   % ************************************************************
   % prepare data: minimum available power levels
   disp(sprintf('   Calculating minimum available power necessary for tag operation...'));
   
   pavopmin.all      = zeros(tagchar_mod{char_ind}.settings.nfch, tagchar_mod{char_ind}.settings.nm);
   pavopmin.square50 = zeros(tagchar_mod{char_ind}.settings.nfch,           1);
   for i = 1 : tagchar_mod{char_ind}.settings.nfch
      % Pav,min vs. frequency and state of modulation
      for j = 1 : tagchar_mod{char_ind}.settings.nm
         pavopmin.all(i,j) = interp1(pic, squeeze(tagchar_mod{char_ind}.m_pav(i, j, :)), tagchar_mod{char_ind}.picopmin(i), 'linear', 'extrap');
      end
   end
   
   
   % ************************************************************
   % MFCW distance errors based on linear model
   % d_err = (phi_d0-phi_d1) * c/(2*w_d);
   disp(sprintf('   Estimating MFCW distance errors based on linear model...'));
   
   % 2FCW distance error; fixed frequency difference vs. center frequency (f0, f0+df)
   %     prepare cell array of matrices
   for k = 1 : length(settings.mfcw.df12)
      linmodel.mfcw.derr_f0{k} = nan(tagchar_mod{char_ind}.settings.nfch, tagchar_mod{char_ind}.settings.np);
   end
   %     calculate
   warning('off', 'MATLAB:interp1:NaNinY');
   warning('off', 'MATLAB:chckxy:IgnoreNaN');
   for i = 1 : tagchar_mod{char_ind}.settings.nfch
      for j = 1 : tagchar_mod{char_ind}.settings.np
         for k = 1 : length(settings.mfcw.df12)
            linmodel.mfcw.derr_f0{k}(i, j) = 3e8./(4*pi*settings.mfcw.df12(k)) * ...
               ( arg.rho_d(i, j) - interp1(fch, arg.rho_d(:, j), fch(i)+settings.mfcw.df12(k), 'linear','extrap') );
         end
      end
   end
   warning('on', 'MATLAB:interp1:NaNinY');
   warning('on', 'MATLAB:chckxy:IgnoreNaN');
   
  
   % ************************************************************
   % plots
   disp(sprintf('   Creating plots...')); plotnames = {}; 
   warning('off', 'MATLAB:legend:UnsupportedFaceColor');
   
   % rho vs f an Pav for Zmod=min and Zmod=max (non-extrapolated)
   %     magnitude
   figure('Visible', settings.figopt.visible); plotnames = [plotnames, 'rho_nonext-mag'];
   subplot(2,1,1); surface(fch/1e9, pav, squeeze(mag.rho(:, end, :))', 'EdgeColor','interp', 'FaceColor','interp');
   hold off; grid on; axis tight; setcolorbar; view([80,45]);
   set(gca, 'YScale','log', 'YTick',settings.pav_tick, 'cLim',xyzlimits(mag.rho(:, [1,end], :)));
   setlabels('MAGNITUDE OF UNMODULATED REFLECTION COEFFICIENT','f [GHz]', 'P_{av} [W]', '|\rho|');
   subplot(2,1,2); surface(fch/1e9, pav, squeeze(mag.rho(:,   1, :))', 'EdgeColor','interp', 'FaceColor','interp'); 
   hold off; grid on; axis tight; setcolorbar; view([80,45]);
   set(gca, 'YScale','log', 'YTick',settings.pav_tick, 'cLim',xyzlimits(mag.rho(:, [1,end], :)));
   setlabels('MAGNITUDE OF MODULATED REFLECTION COEFFICIENT','f [GHz]', 'P_{av} [W]', '|\rho|');
   %     phase
   figure('Visible', settings.figopt.visible); plotnames = [plotnames, 'rho_nonext-phase'];
   subplot(2,1,1); surface(fch/1e9, pav, squeeze(arg.rho(:, end, :))', 'EdgeColor','interp', 'FaceColor','interp');
   hold off; grid on; axis tight; setcolorbar; view([80,45]);
   set(gca, 'YScale','log', 'YTick',settings.pav_tick, 'cLim',xyzlimits(arg.rho(:, [1,end], :)));
   setlabels('PHASE OF UNMODULATED REFLECTION COEFFICIENT IN RAD','f [GHz]', 'P_{av} [W]', 'arg(\rho) [rad]');
   subplot(2,1,2); surface(fch/1e9, pav, squeeze(arg.rho(:,   1, :))', 'EdgeColor','interp', 'FaceColor','interp');
   hold off; grid on; axis tight; setcolorbar; view([80,45]);
   set(gca, 'YScale','log', 'YTick',settings.pav_tick, 'cLim',xyzlimits(arg.rho(:, [1,end], :)));
   setlabels('PHASE OF MODULATED REFLECTION COEFFICIENT IN RAD','f [GHz]', 'P_{av} [W]', 'arg(\rho) [rad]');
   
   % linear model:center rho_c vs f an Pav with fixed color limits for comparison
   figure('Visible', settings.figopt.visible); plotnames = [plotnames, 'linear_center-comp'];
   %     magnitude
   subplot(2,1,1); hold on;
   plot3(fch/1e9, squeeze(pavopmin.all(:,  1)), 1.1*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   plot3(fch/1e9, squeeze(pavopmin.all(:,end)), 1.1*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   surface(fch/1e9, pav, mag.rho_c', 'EdgeColor','none', 'FaceColor','interp');
   grid on; set(gca, 'YScale','log', 'YTick',settings.pav_tick); xlim(xyzlimits(fch)/1e9); ylim([settings.pav_min, settings.pav_max]);
   setlabels('LINEAR MODEL: CENTER VALUE \rho_c (MAGNITUDE) FOR COMPARISON', 'f [GHz]', 'P_{av} [W]', '|\rho_c|');
   setlegend({'P_{min} mod, unmod'}, 'NorthEast'); setcolorbar();
   if ~isempty(settings.clim.rho_c_mag); set(gca, 'CLim', settings.clim.rho_c_mag); end;
   %     phase
   subplot(2,1,2); hold on;
   plot3(fch/1e9, squeeze(pavopmin.all(:,  1)), 370*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   plot3(fch/1e9, squeeze(pavopmin.all(:,end)), 370*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   surface(fch/1e9, pav, arg.rho_c', 'EdgeColor','none', 'FaceColor','interp');
   grid on; set(gca, 'YScale','log', 'YTick',settings.pav_tick); xlim(xyzlimits(fch)/1e9); ylim([settings.pav_min, settings.pav_max]);
   setlabels('LINEAR MODEL: CENTER VALUE \rho_c (PHASE IN RAD) FOR COMPARISON', 'f [GHz]', 'P_{av} [W]', 'arg(\rho_c) [rad]');
   setlegend({'P_{min} mod, unmod'}, 'NorthEast'); setcolorbar();
   if ~isempty(settings.clim.rho_c_arg); set(gca, 'CLim', settings.clim.rho_c_arg); end;
   
   % linear model: difference rho_d vs f an Pav with fixed color limits for comparison
   figure('Visible', settings.figopt.visible); plotnames = [plotnames, 'linear_diff-comp'];
   %     magnitude
   subplot(2,1,1); hold on;
   plot3(fch/1e9, squeeze(pavopmin.all(:,  1)), 1.1*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   plot3(fch/1e9, squeeze(pavopmin.all(:,end)), 1.1*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   surface(fch/1e9, pav, mag.rho_d', 'EdgeColor','none', 'FaceColor','interp');
   grid on; set(gca, 'YScale','log', 'YTick',settings.pav_tick); xlim(xyzlimits(fch)/1e9); ylim([settings.pav_min, settings.pav_max]);
   setlabels('LINEAR MODEL: DIFFERENCE \rho_{\Delta} (MAGNITUDE) FOR COMPARISON', 'f [GHz]', 'P_{av} [W]', '|\rho_{\Delta}|');
   setlegend({'P_{min} mod, unmod'}, 'NorthEast'); setcolorbar();
   if ~isempty(settings.clim.rho_d_mag); set(gca, 'CLim', settings.clim.rho_d_mag); end;
   %     phase
   subplot(2,1,2); hold on;
   plot3(fch/1e9, squeeze(pavopmin.all(:,  1)), 370*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   plot3(fch/1e9, squeeze(pavopmin.all(:,end)), 370*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   surface(fch/1e9, pav, arg.rho_d', 'EdgeColor','none', 'FaceColor','interp');
   grid on; set(gca, 'YScale','log', 'YTick',settings.pav_tick); xlim(xyzlimits(fch)/1e9); ylim([settings.pav_min, settings.pav_max]);
   setlabels('LINEAR MODEL: DIFFERENCE \rho_{\Delta} (PHASE IN RAD) FOR COMPARISON', 'f [GHz]', 'P_{av} [W]', 'arg(\rho_{\Delta}) [rad]');
   setlegend({'P_{min} mod, unmod'}, 'NorthEast'); setcolorbar();
   if ~isempty(settings.clim.rho_d_arg); set(gca, 'CLim', settings.clim.rho_d_arg); end;
   
   % MFCW: distance error for carrier at fch, one secondary carrier at fch+df12 (2FCW)
   figure('Visible', settings.figopt.visible); plotnames = [plotnames, 'mfcw_hist-derr-f0'];
   %     overview 
   subplot(2,1,1); hold on;
   plot3(fch/1e9, squeeze(pavopmin.all(:,  1)), 110*max(max(linmodel.mfcw.derr_f0{1}))*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   plot3(fch/1e9, squeeze(pavopmin.all(:,end)), 110*max(max(linmodel.mfcw.derr_f0{1}))*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   surface(fch/1e9, pav, 100*linmodel.mfcw.derr_f0{1}', 'EdgeColor','none', 'FaceColor','interp');
   hold off; grid on; set(gca, 'YScale','log', 'YTick',settings.pav_tick); xlim(xyzlimits(fch)/1e9); ylim([settings.pav_min, settings.pav_max]);
   setlabels(sprintf('LINEAR MODEL: DISTANCE ERROR [cm] FOR 2FCW, FREQ. DIFFERENCE f_1-f_0 = %.1f MHz', settings.mfcw.df12(1)/1e6),...
      'f_0 [GHz] (main carrier)', 'P_{av} [W]', 'd_{err} [cm]');
   setlegend({'P_{min} mod, unmod'}, 'SouthWest'); setcolorbar();
   if ~isempty(settings.clim.mfcw_derr1); set(gca, 'CLim', 100*settings.clim.mfcw_derr1); end;
   %     zoomed color limits
   subplot(2,1,2); hold on;
   plot3(fch/1e9, squeeze(pavopmin.all(:,  1)), 110*max(max(linmodel.mfcw.derr_f0{1}))*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   plot3(fch/1e9, squeeze(pavopmin.all(:,end)), 110*max(max(linmodel.mfcw.derr_f0{1}))*ones(tagchar_mod{char_ind}.settings.nfch,1), 'Color','k', 'LineWidth',1);
   surface(fch/1e9, pav, 100*linmodel.mfcw.derr_f0{1}', 'EdgeColor','none', 'FaceColor','interp');
   hold off; grid on; set(gca, 'YScale','log', 'YTick',settings.pav_tick); xlim(xyzlimits(fch)/1e9); ylim([settings.pav_min, settings.pav_max]);
   setlabels('[ZOOMED COLOR LIMITS]', 'f_0 [GHz] (main carrier)', 'P_{av} [W]', 'd_{err} [cm]');
   setlegend({'P_{min} mod, unmod'}, 'SouthWest'); setcolorbar();
   if ~isempty(settings.clim.mfcw_derr2); set(gca, 'CLim', 100*settings.clim.mfcw_derr2); end;
   
   
   % ************************************************************
   % save plots to file
   disp(sprintf('   Saving plots...'));
   warning('on', 'MATLAB:legend:UnsupportedFaceColor');
   
   % check if all plots have been named
   if gcf ~= length(plotnames)
      error('Mismatch between number of figures and length of plotnames array.');
   end
   
   % save and close figures
   %   create new directory, meanwhile deactivate warning "Removed '/blah' from the MATLAB path..."
   warning('off', 'MATLAB:RMDIR:RemovedFromPath');
   [status, message, messageid] = rmdir(fullfile(settings.plotfolder, sprintf('tagchar_mod%s_light', tagchar_mod{char_ind}.settings.suffix)), 's'); %#ok<NASGU>
   [status, message, messageid] = mkdir(fullfile(settings.plotfolder, sprintf('tagchar_mod%s_light', tagchar_mod{char_ind}.settings.suffix)));
   warning('on', 'MATLAB:RMDIR:RemovedFromPath');
   %   save figures
   for i = 1 : gcf
      if isempty(plotnames{i})
         savefigure(i, fullfile(settings.plotfolder, sprintf('tagchar_mod%s_light', tagchar_mod{char_ind}.settings.suffix),...
            sprintf('tagchar_mod%s_light_%02i', tagchar_mod{char_ind}.settings.suffix, i)), settings.figopt.type, settings.figopt);
      else
         savefigure(i, fullfile(settings.plotfolder, sprintf('tagchar_mod%s_light', tagchar_mod{char_ind}.settings.suffix),...
            sprintf('tagchar_mod%s_light_%02i_%s', tagchar_mod{char_ind}.settings.suffix, i, plotnames{i})),...
            settings.figopt.type, settings.figopt);
      end
      close(i);
   end
   
end

