% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% batch processing main script for: tag_modulation (reflection coefficient)
%
% WARNING: Settings are not checked for sanity!
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Suitable for Batch-Processing *****
% This module is intended to be used as script (started manually and without the possibility of changing
% settings and/or the behavior). Nontheless it offers a minimalistic interface which enables automated 
% external batch processing. Please refer to the m-file itself for interface definition and behavior,
% as neither interface, nor behavior are fixed.
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
%
% *******************************************************************************************************

function tagchar_modulator_main(v1, v2, simchar)

% *******************************************************************************************************
% initialization
if nargin == 0
   clear; close all; clc;
end

% paths, etc
%     path to globalinit.m
path = fullfile(fileparts(mfilename('fullpath')), '..');
%     Matlab does not resolve symbolic links; if we're running on Unix, let the system resolve them
if isunix
   [dummy, path] = system(sprintf('readlink -f "%s"', path));
end
%     add this path and call globalinit script
restoredefaultpath; addpath(path); clear('dummy', 'path');
globalinit('silent');
% "switch to here"
cd(fileparts(mfilename('fullpath')));

if nargin == 0
   % one last chance to reconsider (only in manual mode)
   if nargin == 0
      disp(sprintf('WARNING: This may take a few hours and might overwrite existing characteristics!\n'));
      reply = input('Are you REALLY sure y/n [n]?  ', 's');
      if ~strcmpi(reply, 'y')
         return
      end
      disp(' ');
   end
end

% prevent full execution by version_system
if get_stacklevel > 0
   return
end


% *******************************************************************************************************
% standard settings (MIGHT BE OVERWRITTEN BELOW)

% TAG CONFIGURATIONS:
%   charid   zic   zmod->repl      za    picmin
%   _b-act    v5   act(v6)         v3      *
%   _b-psv    v5   psv(v7)->psv    v3      *       ... kept passive, max(Picmin) as threshold for repl.
%   _b-p2a    v5   psv(v7)->act    v3      *       ... made  active, max(Picmin) as threshold for repl.
%   _c-act    v5   act(v6)         v6      *
%   _c-psv    v5   psv(v7)->psv    v6      *       ... kept passive, max(Picmin) as threshold for repl.
%   _c-p2a    v5   psv(v7)->act    v6      *       ... made  active, max(Picmin) as threshold for repl.

% general settings
%     basics
if nargin == 0
   settings.simchar  =   true; % suitability for simulator (true) or for theory only (false)
else
   settings.simchar = simchar;
end
settings.writeres =  true; % also write result (characteristic) file or just workspace?
settings.charid   =  '_b-p2a'; %  ID that will be added to filename, start with '_' if not empty
settings.suffix   =    ''; % filename suffix (can be empty); generated automatically below
settings.fs       =   6e9; % Hz sampling frequency
%     system
settings.numthreads    =     1; % maximum number of threads (SLOWER FOR > 1 THREAD)
settings.renew_kticket =  true; % renew Kerberos ticket?
settings.emails        =  true; % send email if task is complete?
%     folders (subfolders)
settings.folders.log =       'logs'; % log (diary)
settings.folders.wsp = 'workspaces'; % workspaceg (for plots, etc)
settings.folders.res =     'tagmod'; % results .mat-file ("tag modulator characteristic")

% data points per dimension
%     base vectors (esp. warping errors will increase dramatically for small values)
if settings.simchar
   settings.nf  = 160; % (old: 192, 384)
   settings.nm  =   6; % (old:  10,  12)
   settings.np  = 128; % (old: 128, 192)
else
   settings.nf  = 192;
   settings.nm  =   2; % light char: only linear model will be used => keep .nm=2
   settings.np  = 256;
end
%     for assembly influence plot (settings.nat x settings.nat)
settings.nat = 250;

% tag characteristics
%     filenames
settings.charfiles.meas_tag_zic    = 'meas_tag_zic';      % chip input impedance
settings.charfiles.meas_tag_zmod   = 'meas_tag_zmod-psv'; % modulation impedance (modulated state)
settings.charfiles.meas_tag_za     = 'meas_tag_za-v3';    % antenna impedance
settings.charfiles.meas_tag_picmin = 'meas_tag_picmin';   % minimum operational chip input power
%     modulation impedance
settings.zmod_max = complex(4e3, -15e3); % "unmodulated" state; modulated is taken from measurements
settings.rmod_min = 2;% Ohm minimum real part of modulation impedance ("prevent active Zmod")
%     special treatment of "passive" zmod characteristics (modulation not possible below min Pic)
%     ... Zic,mod approx Zic,unmod in this case, thus Zmod measurement close to and below min Pic subject
%         to heavy distortions => modulation impedance below picopmin replaced by zmod_max
%     ... threshold='fcn-of-f' will introduce distortions (non-differentiable reflection coefficient) 
%         due to limited resolution in combination with interpolation
%     ... Use picop_tol with care; the char. has the highest resolution close to picmin, thus a high
%         picop_tol shifts the "step" of zmod out of the highres area
settings.psv_zmod.mode      = 'act'; % {'psv': keep passive, 'act': make active}
settings.psv_zmod.threshold = 'max'; % {'max': use max(Picmin), 'fcn-of-f': use Picmin(f)}
settings.psv_zmod.picop_tol =     0; % dB tolerance area above picopmin (set to -Inf not to replace anything)
settings.psv_zmod.flen      =     1; % samples length of smoothing filter to avoid discontinuities

% power characteristics
%     limits for chip input power (extrapolation if necessary)
settings.picmax   = 1e-1; % W
settings.picmin   = 2e-8; % W
%     limits for available power
settings.pavmax   =   10; % W (set high enough to get full char. even for mod)
settings.pavmin   =    0; % W (zero for automatic)
%     assumed "average" path-loss factor (linear indexing for log-dist model with this PLF)
if settings.simchar
   settings.plf = 3;
else
   settings.plf = 6; % light char: bias towards log scale (for ranging_detuning.m)
end
%     settings for iterations to determine optimal pav bounds
settings.npr_inop    = 0.98; % ratio of settings.np withing max(pav) through settings.pavopmin
if settings.simchar
   settings.pavit_maxit = 1000; % maximum number of iterations
   settings.pavit_bias  = 1e-3; % bias for next pav bounds 0<bias<=1 (smaller bias: more exact but slower)
else
   settings.pavit_maxit =  500; % light char: make calculation faster
   settings.pavit_bias  = 5e-2; % light char: make calculation faster
end
settings.npma        = round(settings.np/20); % length of MA filter (smoothing of power vector resolution)
   
% frequency characteristics
%     maximum/minimum frequency
settings.fmin = 1e3; % Hz
settings.fmax = settings.fs/2; % Hz
%     maximum/minimum operation frequency (regulations) => better resolution here
%     (will be adapted to match available frequency grid later)
settings.foptol = 40e6; % better resolution for operation area plus/minus settings.foptol Hz
settings.fopmax = 960e6 + settings.foptol; % Hz
settings.fopmin = 860e6 - settings.foptol; % Hz
%     settings.fopfactor times more data points within area of operation
settings.fopfactor = 2.5;
%     length of MA filter (smoothing of frequency vector resolution)
settings.nfma = round(settings.nf/5);
%     frequency resolution outside characteristic
settings.fres1 = 15; % bins/GHz for zone1 (fmin ... fchmin)
settings.fres4 =  3; % bins/GHz for zone4 (fmax ... fchmax)
%     polynomial models for extrapolation of frequency axis
settings.npoly_zic  = 2; % (unmodulated) chip impedance
settings.npoly_zmod = 2; % modulation impedance

% assembly (serial RC)
settings.cat = 350e-15; % F
%     frequency for Q-matching (Rat-Calculation)
settings.fatm = 915e6; % Hz
%     tolerance (shift) for Rat
settings.rat_shift = 0; % percent
%     min/max for assembly influence plot
settings.rat_min =        1; % Ohm
settings.rat_max =      750; % Ohm
settings.cat_min =    1e-15; % F
settings.cat_max = 2000e-15; % F

% maximum reflection coefficient (rho = 1 is not possible)
settings.rhomax     = 0.999;
settings.rhomax_tol =     1; % percent tolerance for IIR fitting

% random inverse checks rho(f, zmod, pav) -> Zic
if settings.simchar
   settings.checks = 1e4; % number of checks
else
   settings.checks = 1e3; % light char: make calculation faster
end
settings.maxerr = 0.02; % maximum relative mismatch for real/imaginary part
settings.maxavg = settings.maxerr/10; % maximum relative mismatch mean value (real/imaginary part)
%     demilitarized zone around NaN (border region) => no checks here (outliers)
settings.dmz_f = 6; % fch
settings.dmz_p = 6; % pav
settings.dmz_m = 1; % zmod

% IIR bandstop filter parameters to try
%     pronounced V-shape valley
settings.iir{1}.iirorder =      2; % filter order
settings.iir{1}.bshift   =    0.2; % 0...1/2; shift of IIR borders from char. border to center (0: none, 1/2: complete)
settings.iir{1}.cshift   =    0.6; % 0...1;   shift of IIR center towards minimum (0: none, 1: complete)
settings.iir{1}.dfmin    =   50e6; % Hz minimum initial bandwidth of IIR bandstop
%     V-U
settings.iir{2}.iirorder =      3;
settings.iir{2}.bshift   =   0.15;
settings.iir{2}.cshift   =    0.5;
settings.iir{2}.dfmin    =  100e6;
%     U-shape valley
settings.iir{3}.iirorder =      3;
settings.iir{3}.bshift   =   0.25;
settings.iir{3}.cshift   =    0.6;
settings.iir{3}.dfmin    =  150e6;
%     pronounced U-shape valley
settings.iir{4}.iirorder =      3;
settings.iir{4}.bshift   =    0.4;
settings.iir{4}.cshift   =    0.7;
settings.iir{4}.dfmin    =  150e6;

% IIR bandstop optimization (only magnitude is optimized)
%     minimum number of characteristic points to fit IIR
settings.minpts = 2; % < 2 is not allowed (slope detection)
%     options for lsqnonlin
settings.optimset = optimset('Display','off', 'LargeScale','on',...
   'TolX',1e-9, 'TolFun',5e-9, 'MaxIter',1e3, 'MaxFunEval',1e5);
%     initial point x0 [zeros, poles, gain]
settings.optimstart  = [0.98; 0.98; settings.rhomax];
%     bounds for x (upper and lower bound) [zeros, poles, gain]
settings.optimset.ub = [1-100*eps; 1-100*eps; 100]; % stable and minimum-phase
settings.optimset.lb = [0; 0; 0];
%     maximum number of optimizer retries to get |rho|<settings.rhomax
settings.maxoptretries = 2;
%     cost function weights (see function optim_costfcn for explanations)
settings.cf_w = [1, 3, 2, 0.1, 2, 3, 3, 50, 1]';
%     multiplier for weight alteration (will increase IIR residual error but improve fitting quality)
%     (e.g. in case of negative slope => more weight on |rho(f->)|=rhomax and less on slope)
settings.cf_walt = 3.5; % has to be > 1
%     multiplier for |rho|>rhomax punishment (multiplied by this factor for each retry)
settings.cf_wmult = 3; % has to be > 1 ! % 10
%     median filter to create smooth borders after IIR extrapolation
settings.bordermed_len = 5; % samples

% IIR bandstop residual error warning if maximum error above
settings.reserr_warn     = -30; % dB (bad fitting)
settings.reserr_critwarn = -10; % dB (very bad fitting)


% *******************************************************************************************************
% misc

% create directories (return values to suppress warning if directory already exists)
[dummy.stat,dummy.msg,dummy.msgid] = mkdir(settings.folders.log);
[dummy.stat,dummy.msg,dummy.msgid] = mkdir(settings.folders.wsp);
[dummy.stat,dummy.msg,dummy.msgid] = mkdir(settings.folders.res);

% configure email system
global globalsettings;
if settings.emails
   globalsettings.core.mailto      = 'daniel.arnitz@tugraz.at';
   globalsettings.core.subj_prefix = '[CHAR] '; % prefix for subject to identify email in clients
else
   globalsettings.core.mailto      = '';
   globalsettings.core.subj_prefix = '';
end


% *******************************************************************************************************
% define sweep vectors / automatic setup completion

% settings.writeres = false;

if nargin == 0
   % normal operation: Cat + Rat
   %        simulator
   if settings.simchar 
      v1 = [100 : 100 : 800, 1000 : 500 : 3000, 4000, 5000]; % fF Cat
      v2 = [-90, -80, -50, 0, 50, 100, 200, 500, 1000, 2000, 5000]; % percent Rat shift
   %        THEORY ... also make as fast as possible
   else
      % % v1 = [50 : 50 : 1000, 1100 : 100 : 2000, 2200 : 200 : 4000, 4500, 5000]; % fF Cat
      % % v2 = [-98, -95, -90, -80, -67, -50, -33, 0, 50, 100]; % percent Rat shift
      v1 = [50:25:800, 850:50:1000, 1250, 1500, 2000:1000:5000]; % fF Cat
      v2 = [-98, -95, -90, -80, -67, -50, -33, 0, 50, 100, 200, 500, 1000, 2000, 5000]; % percent Rat shift
   end   

   % % IIR filter parameters
   % %     base
   % v1 = 0.2:0.1:0.8; % cshift
   % v2 = 0:0.1:0.4; % bshift
   % v3 = 50e6:50e6:200e6; % dfmin
   % %     combine (non-performant, but easy to understand)
   % v1_full = [];
   % v2_full = [];
   % v3_full = [];
   % for i1 = 1 : length(v1)
   %    for i2 = 1 : length(v2)
   %       v1_full = [v1_full, ones(1,length(v3))*v1(i1)];
   %       v2_full = [v2_full, ones(1,length(v3))*v2(i2)];
   %       v3_full = [v3_full, v3];
   %    end
   % end
   % % [v1_full', v2_full', v3_full'/1e6]
   % v1 = v1_full;
   % v2 = v2_full;
   % v3 = v3_full;
end

% combine sweep vectors
v1_full = sort(repmat(v1, 1, length(v2)));
v2_full =      repmat(v2, 1, length(v1));

% *******************************************************************************************************
% call tagchar_modulator

% for all entries in sweep vectors
for i = 1 : length(v1_full)
   clc; disp(sprintf('%i of %i\n\n', i, length(v1_full))); %#ok<*DSPS>
   
   %    % for IIR filter parameters
   %    settings.suffix = sprintf('_iir2_0350_%04.0f', i);
   %    settings.cshift = v1(i);
   %    settings.bshift = v2(i);
   %    settings.dfmin  = v3(i);
   
   % for normal operation (Cat + Rat)
   settings.cat       = v1_full(i)*1e-15;
   settings.rat_shift = v2_full(i);
   
   % full characteristic (usable by simulator, written in individual files)
   if settings.simchar
      settings.suffix = sprintf('%s-sim_%04.0f_%04.0f', settings.charid, v1_full(i), v2_full(i));
      tagchar_modulator(settings);
      send_email(sprintf('Characteristic tagchar_modulator%s successfully created (%.0f of %.0f)',...
         settings.suffix, i, length(v1_full)), '');
   
   % light characteristic ()
   else
      settings.suffix = sprintf('%s_%04.0f_%04.0f', settings.charid, v1_full(i), v2_full(i));
      tagchar_mod{i}  = tagchar_modulator(settings);
      % extract and delete characteristic string from struct (no setting => identical for all) 
      characteristic = tagchar_mod{i}.characteristic;
      tagchar_mod{i} = rmfield(tagchar_mod{i}, 'characteristic');
   end
end

% write all light characteristics in one file
if ~settings.simchar
   clc; disp(sprintf('\nSaving final results...'));
   % create info header
   matfilename    = sprintf('tagchar_modulator%s-light', settings.charid);
   characteristic = sprintf('%s\n%s',...
      'LIGHT TAG MODULATOR CHARACTERISTIC (NOT SUITABLE FOR SIMULATOR)', characteristic);
   createdon      = datestr(now, 0);
   createdby      = sprintf('%s.m, rev.: %s', mfilename, version);
   usedmatfiles   = 'see individual entries of cell array for details';
   simchar        = false; % not suitable for simulator
   % save and clear workspace (cell array might be huge)
   save(fullfile(settings.folders.res, matfilename), 'matfilename','characteristic','createdby','usedmatfiles',...
         'tagchar_mod', 'v1', 'v2', 'v1_full', 'v2_full');
   clear all;
end

% quit if this was done in batch mode
if nargin > 0
   quit;
end
