% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% resizes a figure ("fill as much space as possible") and saves it to png and/or eps files
%
% 
% Important:
%   = No sanity checks of call parameters are done.
%   = Axes and legends/colorbars have to be created subsequently ("zipper principle").    
%   = If colorbars/legends are used in subplots, each subplot has to have a colorbar/legend.
%   = For colorbars only location "EastOutside" is supported.
%   = Do not maximize figure window before calling this function (resize operation will fail)
%   = The function can be deactivated via a switch: optional.off=true
%
% Comments:
%   - using FOR loops might not be elegant, but spares a lot of if statements, e.g. for
%         - return value of get() = cell (=> cell2mat), vector (=> no cell2mat) or empty
%         - empty legend or colorbar handles
%         - ...
%     plus execution time shouldn't be a problem, because saveas beats everything ;-)
%   - Reported positions can be completely wrong after any resizing operations (more likely on Linux?)
%     There are a number of workarounds in place to deal with these problems.
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Behavior *****
% savefigure(fig, filename, filetype, optional)
%    Resizes figure FIG (maximize used space) and saves the result as graphic FILETYPE (optional; 
%    default: eps plus png) in file FILENAME. Set FILETYPE '' in order to be able to use OPTIONAL and
%    generate the default filetypes.
%
%
% ***** Interface definition *****
% function savefigure(fig, filename, filetype, optional)
%    fig             figure handle
%    filename        filename for graphics
%    filetype        (optional) string {'png', 'pdf', 'eps', 'epsbw', '+fig'=png+eps+fig} default: png+eps
%    optional        (optional) struct with custom settings
%       .papersize         figure size in cm (default: [25, 15.4])
%       .resolution        image resolution in dpi (default: 150)
%       .addspace_xlabel   additional space for x-label 0..1 (from image border; default: 0.01)
%       .addspace_ylabel   additional space for y-label 0..1 (from image border; default: 0.01)
%       .nonconfidential   if true, all tick labels, colorbars and titles will be removed (default: 0)
%       .waittime          seconds wait time before obtaining positions to make sure figure is ready
%                          (default: 0.5)
%       .off               return without saving (naturally false per default)
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
% - shift label text closer to axis in 3D plots
% - checks/support for axes/legends/colorbars not created "zipper-like"
% - support arbitrary position of colorbars
% - colorbar with text on top (colorbar label)
%
% *******************************************************************************************************

function savefigure(fig, filename, filetype, optional)


% *******************************************************************************************************
% internal settings (Matlab figure -> graphic is unfortunatly not WYSIWYG)

% paper size [cm]
internalsettings.papersize = [25, 15.4]; % cm size [width, height] 
internalsettings.sizetol   =       0.05; % size tolerance (relative error)
% subplots
internalsettings.subplotgap = 0.02; % gap between two subplots / 2
% additional space for labels
internalsettings.addspace_xlabel = 0.01; % times plot (axes) height
internalsettings.addspace_ylabel = 0.01; % times plot (axes) width
% factor for addspace_xlabel in case of 3D viewpoints (x: bottom, y: sides)
internalsettings.addspace_x3d = 5;
% legend
internalsettings.legendgap = 0.005; % gap between legend and axes border
internalsettings.lpfactor  =   1.5; % distance legend-axis on one side < by this factor => "inherit" legend
% colorbar
internalsettings.colorbargap = 0.005;  % gap between colorbar and axes border
% figure (re-)size timing problem workarounds
internalsettings.waittime       = 0.5; % s wait time (wait for figure to be complete)
internalsettings.waittimefactor =  10; % wait times the time it took for the set(fig, ...resize) command
% tolerance for position in tiling detection
internalsettings.tilepostol = 0.075;
% image resolution [dpi]
internalsettings.resolution = 150;
% remove "confidential" information (tick labels, title, colorbars) if true
internalsettings.nonconfidential = false;
% return without doing anything (can be used to globally switch of saving of figures)
internalsettings.off = false;


% *******************************************************************************************************
% input parameter checks / prepare parameters
if nargin < 2
   error('Not enough input arguments');
end
if nargin == 2
   filetype = '';
end

if nargin == 4
   % safety check
   if ~isstruct(optional)
      error('Call parameter "optional" has to be a struct');
   end
   % optional fields
   if isfield(optional, 'papersize')
      internalsettings.papersize = optional.papersize;
   end
   if isfield(optional, 'addspace_xlabel')
      internalsettings.addspace_xlabel = optional.addspace_xlabel;
   end
   if isfield(optional, 'addspace_ylabel')
      internalsettings.addspace_ylabel = optional.addspace_ylabel;
   end
   if isfield(optional, 'nonconfidential')
      internalsettings.nonconfidential = optional.nonconfidential;
   end
   if isfield(optional, 'resolution')
      internalsettings.resolution = optional.resolution;
   end
   if isfield(optional, 'waittime')
      internalsettings.waittime = optional.waittime;
   end
   if isfield(optional, 'off')
      internalsettings.off = optional.off;
   end
end

% switched off?
if internalsettings.off
   return;
end


% *******************************************************************************************************
% automatic settings

% aspect ratio (make relative x and y distances identical in created figure)
internalsettings.aspectratio = internalsettings.papersize(1) / internalsettings.papersize(2);


% *******************************************************************************************************
% collect handles

% % make sure figure is ready ... otherwise the reported positions will be wrong
pause(internalsettings.waittime);

% make sure all Axes objects are set to ActivePositionProperty=OuterPosition and Units=normalized for
% resize operation
%     store value of ShowHiddenHandles and make all handles visible
old.root.shh = get(0, 'ShowHiddenHandles');
set(0, 'ShowHiddenHandles', 'on');
%     get all Axes objects in fig
handles.all = findobj(fig, 'Type', 'Axes');
%     and set properties (store old setup)
for i = 1 : length(handles.all)
   % store old setup
   old.axes.activepositionproperty{i} = get(handles.all(i), 'ActivePositionProperty');
   old.axes.units{i}                  = get(handles.all(i), 'Units');
   old.axes.handlevisibility{i}       = get(handles.all(i), 'HandleVisibility');
   % change settings
   set(handles.all(i), 'ActivePositionProperty', 'OuterPosition', 'Units', 'normalized',...
      'HandleVisibility', 'on');
   % make Tag lowercase
   set(handles.all(i), 'Tag', lower(get(handles.all(i), 'Tag')));
end

% get legends and colorbars (have to be tagged beforehand)
handles.legends   = findobj(fig, 'Tag', 'legend');
handles.colorbars = findobj(fig, 'Tag', 'colorbar');

% disable ShowHiddenHandles (to be able to make legends and colorbars invisible)
set(0, 'ShowHiddenHandles', 'off');

% make legends and colorbars invisible for axes search
for i = 1 : length(handles.legends)
   old.legends.handlevisibility{i} = get(handles.legends(i), 'HandleVisibility');
   set(handles.legends(i), 'HandleVisibility', 'off');
end
for i = 1 : length(handles.colorbars)
   old.colorbars.handlevisibility{i} = get(handles.colorbars(i), 'HandleVisibility');
   set(handles.colorbars(i), 'HandleVisibility', 'off');
end

% search for axes
handles.axes = findobj(fig, 'Type', 'axes');

% make legends and colorbars visible again
for i = 1 : length(handles.legends)
   set(handles.legends(i), 'HandleVisibility', old.legends.handlevisibility{i});
end
for i = 1 : length(handles.colorbars)
   set(handles.colorbars(i), 'HandleVisibility', old.colorbars.handlevisibility{i});
end

% restore state of ShowHiddenHandles
set(0, 'ShowHiddenHandles', old.root.shh);


% *******************************************************************************************************
% remove "confidential" information if requested

if internalsettings.nonconfidential
   % remove colorbars
   for i = 1 : length(handles.colorbars)
      delete(handles.colorbars(i));
   end
   handles.colorbars = [];
   % remove titles and tick labels
   for i = 1 : length(handles.axes)
      delete(get(handles.axes(i), 'Title'));
      set(handles.axes(i), 'XTickLabel','', 'YTickLabel','', 'ZTickLabel','');
   end
end


% *******************************************************************************************************
% collect alignment information

% get OuterPosition [left, bottom, width, height] of all ...
outerposition.axes = zeros(length(handles.axes), 4);
outerposition.colorbars = zeros(length(handles.colorbars), 4);
outerposition.legends = zeros(length(handles.legends), 4);
%     ... axes
for i = 1: length(handles.axes)
   outerposition.axes(i, :) =  get(handles.axes(i), 'OuterPosition');
end
%     ... colorbars
for i = 1: length(handles.colorbars)
   outerposition.colorbars(i, :) =  get(handles.colorbars(i), 'OuterPosition');
end
%     ... legends
for i = 1: length(handles.legends)
   outerposition.legends(i, :) =  get(handles.legends(i), 'OuterPosition');
end

% get Position [left, bottom, width, height] of all ...
position.axes = zeros(length(handles.axes), 4);
position.colorbars = zeros(length(handles.colorbars), 4);
position.legends = zeros(length(handles.legends), 4);
%     ... axes
for i = 1: length(handles.axes)
   position.axes(i, :) =  get(handles.axes(i), 'Position');
end
%     ... colorbars
for i = 1: length(handles.colorbars)
   position.colorbars(i, :) =  get(handles.colorbars(i), 'Position');
end
%     ... legends
for i = 1: length(handles.legends)
   position.legends(i, :) =  get(handles.legends(i), 'Position');
end

% get TightInset [left, bottom, right, top] of all ...
tightinset.axes = zeros(length(handles.axes), 4);
tightinset.colorbars = zeros(length(handles.colorbars), 4);
tightinset.legends = zeros(length(handles.legends), 4);
%     ... axes
for i = 1: length(handles.axes)
   tightinset.axes(i, :) =  get(handles.axes(i), 'TightInset');
end
%     ... colorbars
for i = 1: length(handles.colorbars)
   tightinset.colorbars(i, :) =  get(handles.colorbars(i), 'TightInset');
end
%     ... legends
for i = 1: length(handles.legends)
   tightinset.legends(i, :) =  get(handles.legends(i), 'TightInset');
end

% get View [azimuth, elevation}
view.axes = zeros(length(handles.axes), 2);
for i = 1: length(handles.axes)
   view.axes(i, :) =  get(handles.axes(i), 'View');
end

% discern number of subplots (quick/dirty)
tiles.horz = length(minset( outerposition.axes(:,1), internalsettings.tilepostol ));
tiles.vert = length(minset( outerposition.axes(:,2), internalsettings.tilepostol ));
%     check
if length(handles.axes) > tiles.horz * tiles.vert
   error('Detection of subplot alignment (horz, vert) failed.');
end

% check if colorbar position is supported
if ~isempty(outerposition.colorbars) && ...
      any(outerposition.colorbars(:, 3) > outerposition.colorbars(:, 4)) % width > hight?
   error('Positioning of colorbar not supported ... select "EastOutside".');
end

% check amount of ... compared to axes
%     ... colorbars
if ~isempty(handles.colorbars) && length(handles.axes) > length(handles.colorbars)
   error('More axes than colorbars (and at least one colorbar): not supported.')
end
%     ... legends
if ~isempty(handles.legends) && length(handles.axes) > length(handles.legends)
   error('More axes than legends (and at least one legend): not supported.')
end


% *******************************************************************************************************
% resize figure

% % make sure figure is ready ... otherwise the reported positions will be wrong
% pause(internalsettings.waittime);

% % make sure all Axes objects are set to ActivePositionProperty=OuterPosition and Units=normalized for
% % resize operation
% %     store value of ShowHiddenHandles and make all handles visible
% old.root.shh = get(0, 'ShowHiddenHandles');
% set(0, 'ShowHiddenHandles', 'on');
% %     get all Axes objects in fig
% handles.all = findobj(fig, 'Type', 'Axes');
% %     and set properties (store old setup)
% for i = 1 : length(handles.all)
%    % store old setup
%    old.axes.activepositionproperty{i} = get(handles.all(i), 'ActivePositionProperty');
%    old.axes.units{i}                  = get(handles.all(i), 'Units');
%    old.axes.handlevisibility{i}       = get(handles.all(i), 'HandleVisibility');
%    % change settings
%    set(handles.all(i), 'ActivePositionProperty', 'OuterPosition', 'Units', 'normalized',...
%       'HandleVisibility', 'on');
%    % make Tag lowercase
%    set(handles.all(i), 'Tag', lower(get(handles.all(i), 'Tag')));
% end

% resize figure
%     store old units
old.figure.units      = get(fig, 'Units');
old.figure.paperunits = get(fig, 'PaperUnits');
%     change units
set(fig, 'Units', 'centimeters', 'PaperUnits', 'centimeters');
%     store old positions/sizes
old.figure.position       = get(fig, 'Position');
old.figure.papersize      = get(fig, 'PaperSize');
old.figure.paperposition  = get(fig, 'PaperPosition');
%     resize operation
tic;
set(fig,...
   'Units', 'centimeters',...
   'Position', [0,0,internalsettings.papersize],... % +1 to avoid screen border
   'PaperUnits', 'centimeters',...
   'PaperPositionMode', 'manual',...
   'PaperSize', internalsettings.papersize,...
   'PaperPosition', [0,0,internalsettings.papersize]);
t = toc;

% store change in aspect ratio for later usage [width, height]
internalsettings.aspect_change = old.figure.position(3:4) ./ internalsettings.papersize(1:2);

% make sure figure is ready ... otherwise the reported positions will be wrong
pause(max(internalsettings.waittime, t*internalsettings.waittimefactor)); % wait a little

% check if size is okay
pos = get(fig, 'Position');
if abs(pos(3)-internalsettings.papersize(1))/internalsettings.papersize(1) > internalsettings.sizetol ||...
      abs(pos(4)-internalsettings.papersize(2))/internalsettings.papersize(2) > internalsettings.sizetol
   error('Unable to set size of figure properly. Possible problems: figure window maximized or screen resolution too small');
end

% restore old setup
%     axes
for i = 1 : length(handles.all)
   set(handles.all(i), 'ActivePositionProperty', old.axes.activepositionproperty{i},...
      'Units', old.axes.units{i}, 'HandleVisibility',  old.axes.handlevisibility{i});
end
%     figure
set(fig, 'Units', old.figure.units, 'PaperUnits', old.figure.paperunits);


% *******************************************************************************************************
% recalculate outerposition of axes (and partially colorbars)

% prepare arrays
newouterposition.axes = zeros(length(handles.axes), 4);
newouterposition.colorbars = zeros(length(handles.colorbars), 4);

% axes
%  ... subplotgap used for all configurations to make borders for 1x1, 1x2, ..., subplots identical and 
%      as additional space at borders of plot (for labels)
for i = 1: length(handles.axes)
   % from left and lower corner
   newouterposition.axes(i, 1) = round( outerposition.axes(i, 1) * tiles.horz ) / tiles.horz + internalsettings.subplotgap/(2*internalsettings.aspectratio);
   newouterposition.axes(i, 2) = round( outerposition.axes(i, 2) * tiles.vert ) / tiles.vert + internalsettings.subplotgap/2;
   % width and height
   newouterposition.axes(i, 3) = round( outerposition.axes(i, 3) * tiles.horz ) / tiles.horz - internalsettings.subplotgap/internalsettings.aspectratio;
   newouterposition.axes(i, 4) = round( outerposition.axes(i, 4) * tiles.vert ) / tiles.vert - internalsettings.subplotgap;
end

% colorbars (assumes that axis and colorbar handles are aligned)
for i = 1: length(handles.colorbars)
   % keep width, but correct change in aspect ratio caused by modified papersize
   newouterposition.colorbars(i, 3) = outerposition.colorbars(i, 3) * internalsettings.aspect_change(1);
   % shift to right border of axis
   newouterposition.colorbars(i, 1) = newouterposition.axes(i, 1) + newouterposition.axes(i, 3) - newouterposition.colorbars(i, 3);
   % correct width of axis
   newouterposition.axes(i, 3) = newouterposition.axes(i, 3) - newouterposition.colorbars(i, 3) - internalsettings.colorbargap/internalsettings.aspectratio;
end


% *******************************************************************************************************
% recalculate position of axes and (outer)position of colorbars and legends

% prepare arrays
tightnewposition.axes = zeros(length(handles.axes), 4);
newposition.colorbars = zeros(length(handles.colorbars), 4);

% manipulate tightinset of axes and to align subplots
tightinset.axes = repmat(max(tightinset.axes,[],1), length(handles.axes), 1);

% determine multiplication factor for addspace_xlabel (bottom)
addspace_x3d = zeros(length(handles.axes), 1);
for i = 1: length(handles.axes)
   if prod(view.axes(i)) == 0 % this is a flat plot => labels will not be far from axes
      addspace_x3d(i) = 1;
   else
      addspace_x3d(i) = internalsettings.addspace_x3d;
   end
end

% axes
for i = 1: length(handles.axes)
   newposition.axes(i, 1) = newouterposition.axes(i, 1) + tightinset.axes(i, 1) * internalsettings.aspect_change(1);
   newposition.axes(i, 2) = newouterposition.axes(i, 2) + tightinset.axes(i, 2) * internalsettings.aspect_change(2);
   newposition.axes(i, 3) = newouterposition.axes(i, 3) - tightinset.axes(i, 1) * internalsettings.aspect_change(1) - tightinset.axes(i, 3);
   newposition.axes(i, 4) = newouterposition.axes(i, 4) - tightinset.axes(i, 2) * internalsettings.aspect_change(2) - tightinset.axes(i, 4);
   % add additional space for labels
   newposition.axes(i, 1) = newposition.axes(i, 1) + newposition.axes(i, 3) * internalsettings.addspace_ylabel;
   newposition.axes(i, 3) = newposition.axes(i, 3) * (1-internalsettings.addspace_ylabel);
   newposition.axes(i, 2) = newposition.axes(i, 2) + newposition.axes(i, 4) * internalsettings.addspace_xlabel*addspace_x3d(i);
   newposition.axes(i, 4) = newposition.axes(i, 4) * (1-internalsettings.addspace_xlabel*addspace_x3d(i));
end

% colorbars (assumes that axis and colorbar handles are aligned)
for i = 1: length(handles.colorbars)
   % outerposition: vertical arrangement identical to position axis
   newouterposition.colorbars(i, 2) = newposition.axes(i, 2);
   newouterposition.colorbars(i, 4) = newposition.axes(i, 4);
   % position
   newposition.colorbars(i, 1) = newouterposition.colorbars(i, 1);
   newposition.colorbars(i, 2) = newouterposition.colorbars(i, 2);
   newposition.colorbars(i, 3) = newouterposition.colorbars(i, 3) - tightinset.colorbars(i, 3);
   newposition.colorbars(i, 4) = newouterposition.colorbars(i, 4);
end

% legends (assumes that axis and legend handles are aligned)
for i = 1: length(handles.legends)
   % outerposition: shift to position of axis
   %     keep size, but correct change in aspect ratio caused by modified papersize
   newouterposition.legends(i, 3) = outerposition.legends(i, 3) * internalsettings.aspect_change(1);
   newouterposition.legends(i, 4) = outerposition.legends(i, 4) * internalsettings.aspect_change(2);
   %     determine distance of legend borders to axis borders x=[left, right], y=[bottom, top]
   distx = [outerposition.legends(i, 1) - position.axes(i, 1), position.axes(i, 1) + position.axes(i, 3) - outerposition.legends(i, 1) - outerposition.legends(i, 3)];
   disty = [outerposition.legends(i, 2) - position.axes(i, 2), position.axes(i, 2) + position.axes(i, 4) - outerposition.legends(i, 2) - outerposition.legends(i, 4)];
   %     final position: x
   if distx(1) * internalsettings.lpfactor <= distx(2) % much closer to left side: place west
      newouterposition.legends(i, 1) = newposition.axes(i, 1) + internalsettings.legendgap/internalsettings.aspectratio;
   elseif distx(2) * internalsettings.lpfactor <= distx(1) % much closer to right side: place east
      newouterposition.legends(i, 1) = newposition.axes(i, 1) + newposition.axes(i, 3) - newouterposition.legends(i, 3) - internalsettings.legendgap/internalsettings.aspectratio;
   else % distance to both borders approximately equal: place in middle
      newouterposition.legends(i, 1) = newposition.axes(i, 1) + newposition.axes(i, 3)/2 - newouterposition.legends(i, 3)/2;
   end
   %     final position: y
   if disty(1) * internalsettings.lpfactor <= disty(2) % much closer to bottom: place south
      newouterposition.legends(i, 2) = newposition.axes(i, 2) + internalsettings.legendgap;
   elseif disty(2) * internalsettings.lpfactor <= disty(1) % much closer to top: place north
      newouterposition.legends(i, 2) = newposition.axes(i, 2) + newposition.axes(i, 4) - newouterposition.legends(i, 4) - internalsettings.legendgap;
   else % distance to both borders approximately equal: place in middle
      newouterposition.legends(i, 2) = newposition.axes(i, 2) + newposition.axes(i, 4)/2 - newouterposition.legends(i, 4)/2;
   end
   % position (tightinsets not needed)
   newposition.legends(i, 1) = newouterposition.legends(i, 1);
   newposition.legends(i, 2) = newouterposition.legends(i, 2);
   newposition.legends(i, 3) = newouterposition.legends(i, 3);
   newposition.legends(i, 4) = newouterposition.legends(i, 4);
end

% check for negative widths/heights (subplots)
if any(any( newposition.axes(:,3:4) < 0 ))
   error('Unable to place axes with current settings (width/height <= 0)');
end


% *******************************************************************************************************
% apply changes

% axes
for i = 1: length(handles.axes)
   set(handles.axes(i),...
      'OuterPosition', newouterposition.axes(i, :),...
      'Position', newposition.axes(i, :));
end

% colorbars
for i = 1: length(handles.colorbars)
   set(handles.colorbars(i),...
      'OuterPosition', newouterposition.colorbars(i, :),...
      'Position', newposition.colorbars(i, :));
end

% legends
for i = 1: length(handles.legends)
   set(handles.legends(i),...
      'OuterPosition', newouterposition.legends(i, :),...
      'Position', newposition.legends(i, :));
end

% redraw figure (just to make sure) and give Matlab a little time
refresh(fig);
pause(t*internalsettings.waittimefactor);


% *******************************************************************************************************
% save figure

switch lower(filetype)
   case{'png'}
      print(fig, '-dpng', sprintf('-r%i',internalsettings.resolution), filename);
   case{'pdf'}
      print(fig, '-dpdf', sprintf('-r%i',internalsettings.resolution), filename);
   case{'eps'}
      print(fig, '-depsc2', sprintf('-r%i',internalsettings.resolution), filename);
   case{'epsbw'}
      print(fig, '-deps2', sprintf('-r%i',internalsettings.resolution), filename);
   case{'+fig'}   
      print(fig, '-dpng', sprintf('-r%i',internalsettings.resolution), filename);
      print(fig, '-depsc2', sprintf('-r%i',internalsettings.resolution), filename);
      saveas(fig, filename, 'fig');
   otherwise
      print(fig, '-dpng', sprintf('-r%i',internalsettings.resolution), filename);
      print(fig, '-depsc2', sprintf('-r%i',internalsettings.resolution), filename);
end


% *******************************************************************************************************
% *******************************************************************************************************
% *******************************************************************************************************
% returns a vector where no element of SET appears more than once (elements of SET)
function set_el = minset(set, tol)

% vectorize
set = set(:);

% initializes set_el
set_el = [];

for i = 1:length(set)
   % add element to set_el
   set_el = [set_el; set(1)];
   % find element in set
   ind = find( (set >= set(1)-tol).*(set <= set(1)+tol) );
   % remove
   set(ind) = []; %#ok<FNDSB>
   % check if empty
   if isempty(set)
      break;
   end
end 



return
% *******************************************************************************************************
% *******************************************************************************************************
% TEST SCRIPT
% 
% close all
% % XY-PLOTS (legend position approx. subplot position)
% x = linspace(0, 8*pi,1000);
% y = sin(x);
% %     1 x 1
% figure; plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'NorthEast'); setlabels('TITLE', 'x', 'y');
% savefigure(gcf, 'savefigure_test-xy-1x1', 'png');
% %     2 x 1 
% figure;
% subplot(2,1,1); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(t)'}, 'North'); setlabels('TITLE 1', 'x', 'y');
% subplot(2,1,2); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(t)'}, 'South'); setlabels('TITLE 2', 'x', 'y');
% savefigure(gcf, 'savefigure_test-xy-2x1', 'png');
% %     1 x 2 
% figure;
% subplot(1,2,1); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'West'); setlabels('TITLE 1', 'x', 'y');
% subplot(1,2,2); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'East'); setlabels('TITLE 2', 'x', 'y');
% savefigure(gcf, 'savefigure_test-xy-1x2', 'png');
% %     2 x 2 
% figure;
% subplot(2,2,1); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'NorthWest'); setlabels('TITLE 1', 'x', 'y');
% subplot(2,2,2); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'NorthEast'); setlabels('TITLE 2', 'x', 'y');
% subplot(2,2,3); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'SouthWest'); setlabels('TITLE 3', 'x', 'y');
% subplot(2,2,4); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'SouthEast'); setlabels('TITLE 4', 'x', 'y');
% savefigure(gcf, 'savefigure_test-xy-2x2', 'png');
% %     3 x 3 
% figure;
% subplot(3,3,1); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'NorthWest'); setlabels('TITLE 1', 'x', 'y');
% subplot(3,3,2); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'North'); setlabels('TITLE 2', 'x', 'y');
% subplot(3,3,3); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'NorthEast'); setlabels('TITLE 3', 'x', 'y');
% subplot(3,3,4); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'West'); setlabels('TITLE 4', 'x', 'y');
% subplot(3,3,5); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'Best'); setlabels('TITLE 5', 'x', 'y');
% subplot(3,3,6); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'East'); setlabels('TITLE 6', 'x', 'y');
% subplot(3,3,7); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'SouthWest'); setlabels('TITLE 7', 'x', 'y');
% subplot(3,3,8); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'South'); setlabels('TITLE 8', 'x', 'y');
% subplot(3,3,9); plot(x,y); xlim(xyzlimits(x)); ylim(xyzlimits(y));
% setlegend({'sin(x)'}, 'SouthEast'); setlabels('TITLE 9', 'x', 'y');
% savefigure(gcf, 'savefigure_test-xy-3x3', 'png');
% % XYZ-PLOTS
% x = linspace(0, 8*pi,100);
% y = sin(x);
% z = cos(x') * sin(y).^2;
% %     1 x 1
% figure; surf(x,y,z','FaceColor','interp','EdgeColor','none'); xlim(xyzlimits(x)); ylim(xyzlimits(y)); zlim(xyzlimits(z));
% setlegend({'cos(x) * sin(y)^2'}, 'NorthWest'); setcolorbar(); setlabels('TITLE', 'x', 'y', 'z');
% savefigure(gcf, 'savefigure_test-xyz-1x1', 'png');
% %     2 x 2
% figure; 
% subplot(2,2,1); surf(x,y,z','FaceColor','interp','EdgeColor','none'); xlim(xyzlimits(x)); ylim(xyzlimits(y)); zlim(xyzlimits(z));
% setlegend({'cos(x) * sin(y)^2'}, 'NorthWest'); setcolorbar(); setlabels('TITLE 1', 'x', 'y', 'z');
% subplot(2,2,2); surf(x,y,z','FaceColor','interp','EdgeColor','none'); xlim(xyzlimits(x)); ylim(xyzlimits(y)); zlim(xyzlimits(z));
% setlegend({'cos(x) * sin(y)^2'}, 'NorthEast'); setcolorbar(); setlabels('TITLE 2', 'x', 'y', 'z');
% subplot(2,2,3); surf(x,y,z','FaceColor','interp','EdgeColor','none'); xlim(xyzlimits(x)); ylim(xyzlimits(y)); zlim(xyzlimits(z));
% setlegend({'cos(x) * sin(y)^2'}, 'SouthWest'); setcolorbar(); setlabels('TITLE 3', 'x', 'y', 'z');
% subplot(2,2,4); surf(x,y,z','FaceColor','interp','EdgeColor','none'); xlim(xyzlimits(x)); ylim(xyzlimits(y)); zlim(xyzlimits(z));
% setlegend({'cos(x) * sin(y)^2'}, 'SouthEast'); setcolorbar(); setlabels('TITLE 4', 'x', 'y', 'z');
% savefigure(gcf, 'savefigure_test-xyz-2x2', 'png');
