% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% channel - main (time-invariant, XIXO)
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Behavior *****
% version = channel_main()
%    Just returns the version number (string).
% output = channel_main(input, settings)
%    Applies a largescale, directivity, and smallscale channel model, adds noise at the receivers, and
%    returns the result in OUTPUT. This function supports and arbitrary number of inputs and outputs
%    (XIXO).
%    The vectors in INPUT can be of arbitrary length; the length of all vectors in OUTPUT is identical
%    and equal to the maximum vector length in INPUT. The function assumes that noise settings depend
%    only on the receiver (identical across all transmitters for one receiver).
%
%    Note that the function sets globalsettings.core.mat_stubborn temporarily to true for loading of
%    sparse FIR performance characteristic.
% 
%    See also CHANNEL_LARGE, CHANNEL_DIRECTIVITY, CHANNEL_SMALL, and CHANNEL_NOISE.
%
%
% ***** Global Variables *****
% globalsettings
%    .core.sfir_char       filename for sparse FIR performance characteristic
%    .core.mat_stubborn    ignore missing files; return empty struct if file is missing
%
%
% ***** Interface definition *****
% function output = channel_main(input, settings)
%    input       input vector cell array (one cell per transmitter)
%    settings    struct containing all necessary parameters
%       .direct          (optional) struct: predefined large-scale gain/delay
%          .gain            gain factor
%          .delay_s         delay in samples
%       .largescale      struct containing large-scale model parameters
%          .type            type of large-scale model {'log-dist'}
%          .dist            distance/distances in m (supports up to two dimensions)
%          .pl              path-loss factor
%          .c               speed of light in m/s
%          .f0              "center" frequency/frequencies for attenuation in Hz
%          .fs              sampling frequency in Hz
%       .directivity     struct containing antenna directivity model parameters
%          .txant           filename of TX antenna characteristic (isotropic if empty)
%          .txrot           rotation of TX antenna [azimuth, elevation] ([0,0]: maximum in positive x-axis)
%          .rxant           filename of RX antenna characteristic (isotropic if empty)
%          .rxrot           rotation of RX antenna [azimuth, elevation] ([0,0]: maximum in positive x-axis)
%          .dir             direction of channel [azimuth, elevation] (from TX antenna)
%       .reflections    struct containing substructs with definitions of one reflection each
%          .example        one such substruct, for example (e.g., definition of a wall)
%             .largescale     large-scale model parameters for this reflection; see settings.largescale
%             .directivity    antenna directivity model parameters for this reflection; see settings.directivity
%             .n2             refractive index of reflective surface (typ n2>1)
%             .dim            dimension of the surface's normal vector (e.g. 3 means z in [x,y,z]) 
%             .pol_dim        dimension of polarization vector (electric field)  (e.g. 3 means z in [x,y,z]) 
%             .aoi            angle of incidence (0<=aoi<=90) in degree
%       .smallscale      struct containing small-scale model parameters
%          .on              switch small-scale model on/off
%          .det             return the average PDP (true) or random NLOS paths with the same average PDP (false)
%          .seed            random seed for non-deterministic setup (set to NaN for normal operation)
%          .ensembles       number of independent ensembles for settings.det=false
%          .maxrays         maximum number of paths (not including the LOS path, i.e. maxrays=0 means LOS only)
%          .maxiter         maximum iterations for getting the rms delay spread right
%          .k               Ricean K-factor in dB
%          .trms            RMS delay spread in s
%          .bw              bandwidth for channel (with oversampling) in Hz
%          .fres            initial frequency resolution in Hz (might be increased to meet .trms)
%          .eps_k           relative tolerable error in K-factor (linear)
%          .eps_trms        relative tolerable error in RMS delay spread
%          .fs              sampling frequency in Hz
%       .noise            struct containing noise parameters
%          .type            type of noise {'wgn' white Gaussian, otherwise: no noise}
%                           (use 'off' to skip the call of CHANNEL_NOISE ... faster)
%          .n0              noise density [dBm, per ? Hz], e.g., [-50, 1000] = -50 dBm/kHz
%          .frxs            eceiver sampling frequency in Hz
%          .fs              sampling frequency in Hz
%
%    output            output vector cell array (one cell per receiver)
%    smallscale_stat   struct STATISTICS returned by CHANNEL_SMALL if smallscale model is on, struct 
%                      without fields otherwise
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
% - ultrawideband largescale channel
% ? mute version_system for large amount of channels (get versions only once)
%
% *******************************************************************************************************


function [output, smallscale_stat] = channel_main(input, settings)
version = 'beta 1.0';


% *******************************************************************************************************
% version system

% just return version number
if nargin == 0
   output = version;
   return
end

% call version system (logs version numbers)
version_system();


% *******************************************************************************************************
% internal settings

% warnings and messages
internalsettings.maxpaths_warn    = 250; % maximum number of (overall) paths before a warning is issued

% sparsity before loop implementation is selected (in case n performance characteristic is available)
internalsettings.default_xing_pt = 0.98; % <=0: always loop, >=1: always filter


% *******************************************************************************************************
% input parameter checks / prepare paramters

global globalsettings; % for sparse FIR performance characteristic

% number of input/output signals
ntx = size(settings, 1);
nrx = size(settings, 2);

% initialize smallscale parameter cell array
smallscale_stat = cell(ntx,nrx);
smallscale_stat = cellfun(@(x) struct(), smallscale_stat, 'UniformOutput',false);

% length of output signal => maximum length of all input signals
nout = max(cellfun(@length, input));
if nout == 0
   warn('No signals to transmit. Returning empty vectors (required length unknown).');
   output = cell(1, nrx);
   return;
end

% for all settings
for i = 1 : ntx
   for j = 1 : nrx
      % check contents of settings (substructs are needed => check everything)
      %     prepare required data
      expected.name = sprintf('settings{%.0f,%.0f}', i, j);
      expected.reqfields = {...
         'smallscale', {'on','det','seed','ensembles','maxrays','k','trms','eps_k','eps_trms','bw','fres','fs'},...
         'noise', {'type', 'n0', 'frxs', 'fs'}};
      % add either largescale+directivity or direct path setups;
      % ... check if both setups are present (might easily lead to an error)
      if isfield(settings{i,j}, 'direct') && ( isfield(settings{i,j}, 'largescale') || isfield(settings{i,j}, 'directivity') )
         err('Setup contains direct path and largescale/directivity settings. This is not allowed.');
      elseif isfield(settings{i,j}, 'direct')
         expected.reqfields = [expected.reqfields, {'direct', {'gain', 'delay_s'}}];
      elseif ( isfield(settings{i,j}, 'largescale') || isfield(settings{i,j}, 'directivity') )
         expected.reqfields = [expected.reqfields, {...
            'largescale', {'type', 'dist', 'pl', 'c', 'f0', 'fs'},...
            'directivity', {'txant','txrot','txant','txrot','dir_tx'}}];
      end
      %     reflective surfaces
      if isfield(settings{i,j}, 'reflections')
         refl_names = fieldnames(settings{i,j}.reflections);
         refl_cells = cell(0);
         for k = 1 : length(refl_names)
            refl_cells = [refl_cells, {refl_names{k}, {'n2', 'dim', 'pol_dim', 'aoi', ...
               'largescale', {'type', 'dist', 'pl', 'c', 'f0', 'fs'},...
               'directivity', {'txant','txrot','txant','txrot','dir_tx'}}}];
         end
         expected.reqfields = [expected.reqfields, {'reflections', refl_cells}];
         clear('refl_names', 'refl_cells');
      end
      %     check
      errortext = contentcheck(settings{i,j}, expected);
      %     output
      if ~isempty(errortext)
         err('Incomplete settings{%.0f,%.0f}\n%s', i, j, errortext);
      end
   end
end


% *******************************************************************************************************
% channel gain, multipath propagation (small-scale and large-scale model)

% get delays (in samples) and attenuations
for i = 1 : ntx
   if isempty(input{i}); continue; end % skip inactive transmitters
   for j = 1 : nrx
      
      % large-scale (direct channel has to REPLACE largescale model + directivity; DO NOT COMBINE!)
      if isfield(settings{i,j}, 'direct') % direct channel (results already provided)
         delay_ls = settings{i,j}.direct.delay_s;
         gain_ls  = settings{i,j}.direct.gain;
         gain_dir = 1;
      else % normal model
         % largescale model
         [delay_ls, gain_ls] = channel_large(settings{i,j}.largescale);
         % directivity (antenna radiation patterns)
         gain_dir = channel_directivity(settings{i,j}.directivity);
      end
      
      % reflective surfaces
      if isfield(settings{i,j}, 'reflections')
         refl_names = fieldnames(settings{i,j}.reflections);
         delay_rfl = zeros(length(refl_names), 1);
         gain_rfl  = zeros(length(refl_names), 1);
         for k = 1 : length(refl_names)
            [delay_rfl(k), gain_rfl(k)] = channel_large(settings{i,j}.reflections.(refl_names{k}).largescale);
            gain_rfl(k) = gain_rfl(k) * channel_directivity(settings{i,j}.reflections.(refl_names{k}).directivity);
            gain_rfl(k) = gain_rfl(k) * channel_reflection(settings{i,j}.reflections.(refl_names{k}));
         end
      end
      
      % small-scale
      if settings{i,j}.smallscale.on
         [delay_ss, gain_ss, smallscale_stat{i,j}] = channel_small(settings{i,j}.smallscale);
      else
         delay_ss = 0;
         gain_ss  = 1;
      end  
      
      % combine largescale, smallscale, and directivity models
      delay_s{i,j} = delay_ls + delay_ss;
      gain{i,j} = gain_ls * gain_dir * gain_ss;
      
      % add reflections (if present)
      if isfield(settings{i,j}, 'reflections')
         delay_s{i,j} = [delay_s{i,j}; delay_rfl];
         gain{i,j}    = [gain{i,j}; gain_rfl];
         % sort delays
         [delay_s{i,j}, sort_ind] = sort(delay_s{i,j});
         gain{i,j} = gain{i,j}(sort_ind);
         % make sure there are no identical delays
         if any(diff(delay_s{i,j})==0)
            for k = 1 : length(delay_s{i,j}) - 1
               if delay_s{i,j}(k) == delay_s{i,j}(k+1)
                  gain{i,j}(k+1) = gain{i,j}(k+1) + gain{i,j}(k);
                  gain{i,j}(k)   = NaN; % mark as obsolete
               end
            end
            %     remove obsolute entries
            delay_s{i,j}(isnan(gain{i,j})) = [];
            gain{i,j}(isnan(gain{i,j})) = [];
         end
      end
      
   end
end

% output number of paths (only if at least one transmitter is active)
%     number of paths
if ntx > 1 || nrx > 1
   len = cellfun(@length, delay_s);
   message('Considering the following amount of paths per channel transmitter -> receiver:');
   message('        %s', sprintf('rx%02.0f ', [1:1:nrx]));
   for i = 1 : ntx
      if isempty(input{i}); continue; end % skip inactive transmitters
      message('  tx%02.0f  %s', i, sprintf('%4.0f ', len(i,:)));
   end
else
   message('Considering %.0f paths per channel transmitter -> receiver.', length(delay_s{1,1}));
end
%     warning for large amount of paths
if sum(sum(cellfun(@length, delay_s))) > internalsettings.maxpaths_warn
   warn('Large amount of paths to consider; this may take a while.');
end

% load performance characteristic to select fastest filtering method
%  ... make sure "stubborn matfile loading" (ignore missing files) is enabled
mat_stubborn_temp = globalsettings.core.mat_stubborn;
globalsettings.core.mat_stubborn = true;
pchar = loadmat(globalsettings.core.sfir_char);
globalsettings.core.mat_stubborn = mat_stubborn_temp;
pchar.exists = ~isempty(fieldnames(pchar));
if ~pchar.exists % no performance char file exists for this host
   warn('No sparse FIR performance found for this host. Switching to defaults.');
   pchar.xing_pt = internalsettings.default_xing_pt;
end

% apply delay and gain
for j = 1 : nrx
   output{j} = zeros(nout, 1);
   for i = 1 : ntx
      % input signal length
      len = length(input{i});
      if len == 0; continue; end % skip inactive transmitters
      
      % get performance crossing between filter and loop implementation (if available)
      if pchar.exists
         ind_i = interp1(pchar.len_i_log10, pchar.ind_i, log10(len), 'nearest', 'extrap');
         ind_f = interp1(pchar.len_f_log10, pchar.ind_f, log10(delay_s{i,j}(end)), 'nearest', 'extrap');
         pchar.xing_pt = pchar.sxing(ind_i, ind_f);
      end
      
      % filtering
      if (delay_s{i,j}(end)-length(delay_s{i,j}))/delay_s{i,j}(end) > pchar.xing_pt % loop faster (low density)
         for k = 1 : length(delay_s{i,j}) % for all paths (assumed to be sparse => didn't use filter)
            output{j}(1+delay_s{i,j}(k):len) =...
               output{j}(1+delay_s{i,j}(k):len) + input{i}(1:end-delay_s{i,j}(k)) * gain{i,j}(k);
         end
      else
         gain_full = zeros(1+delay_s{i,j}(end), 1); % first term in numerator of FIR corresponds to delay_s=0
         gain_full(1+delay_s{i,j}) = gain{i,j};
         output{j}(1:len) = output{j}(1:len) + filter(gain_full, 1, input{i});
      end
   end
end


% *******************************************************************************************************
% noise
%  ... using cell2mat / mat2cell and the ability of channel_noise to process multiple receivers at
%      once is slower for small amounts of receivers => treat all receivers separately

for j = 1 : nrx
   if ~strcmpi(settings{j}.noise.type, 'off')
      output{j} = channel_noise(output{j}, settings{1,j}.noise); % identical for all transmitters
   end
end

