% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% channel - location of readers/tags has changed => recalculate distances and setup channels accordingly
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Behavior *****
% version = channel_newpos()
%    Just returns the version number (string).
% settings = channel_newpos(settings, pos_t, pos_r, ownsettings)
%    Modifies and returns the channel setup given in SETTINGS. Positions of transmitters and receivers
%    have to be provided in matrices/cell arrays POS_T and POS_R, respectively. Azimuth is set to 0 deg
%    and elevation is set to 90 deg by default (e.g., for 1-dim positions).
%    Smallscale settings are modified using linear interpolation and according to the vectors
%    given in OWNSETTINGS, for example:
%       ownsettings.v_dist = [ 0, 1, 2, 3]; ownsettings.v_k = [10, 9, 7, 0]; 
%       ownsettings.v_trms = [ 2, 4, 6, 8]; pos_t = {0, 1};  pos_r = {2, 3};  
%          => distance = [2, 1; 3, 2]
%          => Ricean K-factors = {7, 9; 0, 7};
%          => RMS delay spread = {6, 4; 8, 6};     
%    Reflective surfaces are added according to OWNSETTINGS.REFLECTIONS (if present). The reflected (NLOS) 
%    paths inherit all basic largescale and directivity properties (settings) of the direct (LOS) path.
%    
%
%
% ***** Interface definition *****
% function settings = channel_newpos(settings, pos_t, pos_r, ownsettings)
%    settings       channel settings to be modified (largescale, smallscale, directivity and reflections)
%    pos_t          position of  N transmitters in meters; 
%                   e.g. 3-dim [x1, y1, z1; x2, y2, z2; ...; xN, yN, zN] or cell array of vectors 
%    pos_r          position of M receiver in meters; 
%                   e.g. 3-dim [x1, y1, z1; x2, y2, z2; ...; xN, yN, zN] or cell array of vectors 
%    ownsettings    struct containing settings for this function
%       .v_dist        distance reference vector in meters
%       .v_k           vector for Ricean K-factor "K = v_k(v_dist)" according to CHANNEL_SMALL
%       .v_trms        vector for RMS delay spread "trms = v_trms(v_dist)" according to CHANNEL_SMALL
%       .reflections   (optional) struct containing substructs with definitions of one reflective surface each
%          .example       one such substruct, for example
%             .dim           dimension normal to reflective surface (e.g. 3 in [x,y,z] if xy reflects)
%             .shift         shift of this surface along the normal vector
%
%    settings   modified largescale, smallscale, and reflection settings settings
%    
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
% ? allow arbitrary reflective surfaces (including bounds)
% ? recalculate only modified positions (performance)
%
% *******************************************************************************************************


function settings = channel_newpos(settings, pos_t, pos_r, ownsettings)
version = 'beta 1.0';


% *******************************************************************************************************
% version system

% just return version number
if nargin == 0
   settings = version;
   return
end

% call version system (logs version numbers)
version_system();


% *******************************************************************************************************
% input parameter checks / prepare parameters

% check contents of settings
%     prepare required data
expected.name = 'ownsettings';
expected.reqfields = {'v_dist', 'v_k', 'v_trms'};
%        reflective surfaces
if isfield(ownsettings, 'reflections')
   expected.reqfields = [expected.reqfields, 'reflections'];
   refl_names = fieldnames(ownsettings.reflections);
   refl_cells = cell(0);
   for k = 1 : length(refl_names)
      refl_cells = [refl_cells, {refl_names{k}, {'dim', 'shift'}}];
   end
   expected.reqfields = [expected.reqfields, {'reflections', refl_cells}];
   clear('refl_names', 'refl_cells');
end
%     check
errortext = contentcheck(ownsettings, expected);
%     output
if ~isempty(errortext)
   err('Incomplete settings\n%s', errortext);
end

% distance vector in ownsettings has to contain d=0 (otherwise saturation below won't work)
if min(ownsettings.v_dist) ~= 0  
   err('Distance vector in smallscale setup (call param. ownsettings) has to be positive at start at zero.');
end


% *******************************************************************************************************
% modify given setup

% calculate new distances and angles
[dist_tr, az_tr, el_tr] = get_distances(pos_t, pos_r);

% calculate distance for 1-point reflections (if there is at least one reflective surface)
if isfield(ownsettings, 'reflections')
   % for all defined reflective surfaces
   refl_names = fieldnames(ownsettings.reflections);
   for k = 1 : length(refl_names)
      % mirrored positions
      %     transmitter
      pos_tm = pos_t;
      if iscell(pos_tm)
         for j = 1 : length(pos_tm) % [?] cellfun
            pos_tm{j}(ownsettings.reflections.(refl_names{k}).dim) = ...
               2*ownsettings.reflections.(refl_names{k}).shift - pos_tm{j}(ownsettings.reflections.(refl_names{k}).dim);
         end
      else
         pos_tm(:, ownsettings.reflections.(refl_names{k}).dim) = ...
            repmat(2*ownsettings.reflections.(refl_names{k}).shift,size(pos_tm,1),1) - ...
            pos_tm(:, ownsettings.reflections.(refl_names{k}).dim);
      end
      %     receiver
      pos_rm = pos_r;
      if iscell(pos_rm)
         for j = 1 : length(pos_rm) % [?] cellfun
            pos_rm{j}(ownsettings.reflections.(refl_names{k}).dim) = ...
               2*ownsettings.reflections.(refl_names{k}).shift - pos_rm{j}(ownsettings.reflections.(refl_names{k}).dim);
         end
      else
         pos_rm(:, ownsettings.reflections.(refl_names{k}).dim) = ...
            repmat(2*ownsettings.reflections.(refl_names{k}).shift,size(pos_rm,1),1) - ...
            pos_rm(:, ownsettings.reflections.(refl_names{k}).dim);
      end
      
      % get distances and angles
      [dist_tmr{k}, az_tmr{k}, el_tmr{k}] = get_distances(pos_tm, pos_r);
      [dist_trm{k}, az_trm{k}, el_trm{k}] = get_distances(pos_t, pos_rm);
   end
end

% warn if any distance is outside ownsettings.v_dist
if any(any(dist_tr > max(ownsettings.v_dist)))
   critwarn('Distances outside range given in ownsettings. Saturating trms, using linear extrap. for K.');
end

% for all combinations
for i = 1 : size(dist_tr, 1)
   for j = 1 : size(dist_tr, 2)
      
      % largescale and directivity model (if largescale model exists)
      if isfield(settings{i,j}, 'largescale')
         settings{i,j}.largescale.dist = dist_tr(i,j); % for all dimensions
         if ~isempty(az_tr)
            settings{i,j}.directivity.dir_tx(1) = az_tr(i,j) * 180/pi; % only for 2d,3d
         else
            settings{i,j}.directivity.dir_tx(1) = 0;
         end
         if ~isempty(el_tr)
            settings{i,j}.directivity.dir_tx(2) = el_tr(i,j) * 180/pi; % only for 3d
         else
            settings{i,j}.directivity.dir_tx(2) = 90; % default: in xy-plane
         end
      end
      
      % reflection model if exists
      if isfield(ownsettings, 'reflections')
         % a reflective surfaces without a propagation model does not make much sense
         if ~isfield(settings{i,j}, 'largescale')
            critwarn('The channel has reflective surfaces, but no largescale model; ignoring the surfaces.');
         end
         
         refl_names = fieldnames(ownsettings.reflections);
         for k = 1 : length(refl_names)
            % copy largescale, directivity settings
            settings{i,j}.reflections.(refl_names{k}).largescale  = settings{i,j}.largescale;
            settings{i,j}.reflections.(refl_names{k}).directivity = settings{i,j}.directivity;
            % modify settings
            settings{i,j}.reflections.(refl_names{k}).largescale.dist = dist_tmr{k}(i,j);
            if ~isempty(az_tr) % 2d,3d
               settings{i,j}.reflections.(refl_names{k}).directivity.dir_tx(1) = az_trm{k}(i,j) * 180/pi; % TX looks at the mirrored image of RX
               settings{i,j}.reflections.(refl_names{k}).directivity.dir_rx(1) = az_tmr{k}(i,j) * 180/pi; % ... and vice-versa
            else
               settings{i,j}.reflections.(refl_names{k})(1).directivity.dir_tx(1) = 0;
            end
            if ~isempty(el_tr) % 3d
               settings{i,j}.reflections.(refl_names{k}).directivity.dir_tx(2) = el_trm{k}(i,j) * 180/pi; % TX looks at the mirrored image of RX
               settings{i,j}.reflections.(refl_names{k}).directivity.dir_rx(2) = el_tmr{k}(i,j) * 180/pi; % ... and vice-versa
            else
               settings{i,j}.reflections.(refl_names{k}).directivity.dir_tx(2) = 90; % default: in xy-plane
            end
            % calculate angle of incidence on surface
            switch ownsettings.reflections.(refl_names{k}).dim
               case 1 % "x => yz-plane reflects"
                  alpha = settings{i,j}.reflections.(refl_names{k}).directivity.dir_tx(1);
               case 2 % "y => xz-plane reflects"
                  alpha = settings{i,j}.reflections.(refl_names{k}).directivity.dir_tx(1) - 90;
               case 3 % "z => xy-plane reflects"
                  alpha = settings{i,j}.reflections.(refl_names{k}).directivity.dir_tx(2);
               otherwise
                  err('Unsupported normal vector dimension "%i" for reflective surface.', ownsettings.reflections.(refl_names{k}).dim);
            end
            settings{i,j}.reflections.(refl_names{k}).aoi = abs(round(alpha/180)*180 - alpha);
         end
      end
      
      % smallscale model
      settings{i,j}.smallscale.k    = interp1(ownsettings.v_dist, ownsettings.v_k,    dist_tr(i,j),...
         'linear', 'extrap');
      settings{i,j}.smallscale.trms = interp1(ownsettings.v_dist, ownsettings.v_trms, dist_tr(i,j),...
         'linear', ownsettings.v_trms(end));
   end
end
