% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% channel - reflective surface
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Behavior *****
% version = channel_large()
%    Just returns the version number (string).
% gain = channel_reflection(settings)
%    Calculates the reflection coefficient (amplitude gain) of a flat surface using the Fresnel
%    equations. The incident wave can either be polarized parallel or perpenticular to the surface, mixed
%    polarizations are not considered. The function furthermore assumes a refractive index of n1=1 (air) 
%    for the main transmission medium (can be changed via internalsettings within this function).
%    
%
%
% ***** Interface definition *****
% gain = channel_reflection(settings)
%    settings   struct containing all necessary parameters
%       .n2        refractive index of second material (typ n2>1)
%       .dim       dimension of the surface's normal vector (e.g. 3 means z in [x,y,z]) 
%       .pol_dim   dimension of polarization vector (electric field)  (e.g. 3 means z in [x,y,z]) 
%       .aoi       angle of incidence (0<=aoi<=90) in degree; can be a multidimensional array
%                  (the aoi is measured between the wave's direction of movement and the surface normal)
%
%    gain        reflection coefficient (amplitude gain)
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
%
% *******************************************************************************************************


function gain = channel_reflection(settings)
version = 'beta 1.0';


% *******************************************************************************************************
% version system

% just return version number
if nargin == 0
   gain = version;
   return
end

% call version system (logs version numbers)
version_system();


% *******************************************************************************************************
% internal settings

internalsettings.n1 = 1; % refractive index of main transmission medium (air: n=1)


% *******************************************************************************************************
% input parameter checks / prepare parameters

% check contents of settings
%     prepare required data
expected.name = 'settings';
expected.reqfields = {'n2', 'dim', 'pol_dim', 'aoi'};
%     check
errortext = contentcheck(settings, expected);
%     output
if ~isempty(errortext)
   err('Incomplete settings\n%s', errortext);
end

% check aoi
if any(settings.aoi(:) < 0) || any(settings.aoi(:) > 90)
   err('Angle of incidence (settings.aoi) has to be 0<=aoi<=90 degree.');
end


% *******************************************************************************************************
% calculate reflection coefficient from Fresnel equations

% a few shortcuts
a  = settings.aoi * pi/180;
n1 = internalsettings.n1;
n2 = settings.n2;

% polarization perpenticular to reflective surface
if settings.dim == settings.pol_dim
   gain = (-n2*cos(a)+n1*sqrt(1-(n1/n2*sin(a)).^2) ) ./ ( n2*cos(a)+n1*sqrt(1-(n1/n2*sin(a)).^2) );
   
% polarization along (parallel to) reflective surface
else
   gain = ( n1*cos(a)-n2*sqrt(1-(n1/n2*sin(a)).^2) ) ./ ( n1*cos(a)+n2*sqrt(1-(n1/n2*sin(a)).^2) );
end
