% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% channel - small-scale (time-invariant)
%
% references:
% [1] K. Witrisal, A New Method to Measure Parameters of Frequency-Selective Ration Channels Using Power
%     Measurements, IEEE Transactions on Communications, Vol. 59, No 10, October 2001, pp. 1788-1800
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Behavior *****
% version = channel_small()
%    Just returns the version number (string).
% [delay_s, gain, statistics] = channel_small(settings)
%    Returns gain as a function of delay (in samples) using the power-delay-profile (PDP) model in [1]. 
%    The function tries to generate the an average power-delay-profile according to settings. The final
%    results are stored in STATISTICS. The function returns a perfect average power-delay-profile 
%    if SETTINGS.DET is set to true. For SETTINGS.DET==false SETTINGS.ENSEMBLES independent (random)
%    ensembles are generated. Note that the delay axis is also randomized in this case.
%
%
% ***** Interface definition *****
% function [delay_s, gain, statistics] = channel_small(settings)
%    settings    struct containing all necessary parameters
%       .on          switch small-scale model on/off
%       .det         return the average PDP (true) or random NLOS paths with the same average PDP (false)
%       .seed        random seed for non-deterministic setup (set to NaN for normal operation)
%       .ensembles   number of independent ensembles for settings.det=false
%       .maxrays     maximum number of paths (not including the LOS path, i.e. maxrays=0 means LOS only)
%       .nrays_f     (optional) force number of paths (not including the LOS path)
%       .maxiter     maximum iterations for getting the rms delay spread right
%       .k           Ricean K-factor in dB
%       .trms        RMS delay spread in s
%       .bw          bandwidth for channel (with oversampling) in Hz
%       .fres        initial frequency resolution in Hz (might be increased to meet .trms)
%       .eps_k       relative tolerable error in K-factor (linear)
%       .eps_trms    relative tolerable error in RMS delay spread
%       .fs          sampling frequency in Hz
%
%   delay_s      delays for the different paths in samples (lines are ensembles for settings.det=false)
%   gain         gain for the different paths (lines are ensembles for settings.det=false)
%   statistics   struct containing estimated statistics of the generated average PDP and warning switches
%      .av_k           estimated Ricean K-factor in dB
%      .av_trms        estimated RMS delay spread
%      .iter           number of iterations needed to meet settings.eps_k
%      .n              number of paths
%      .fres           final frequency resolution of the channel
%      .bw             final bandwidth of the channel
%      .warn_res       warning: there might be problems with the resulting frequency resolution
%      .warn_bw        warning: there might be problems with the resulting bandwidth
%      .warn_ktrms     warning:  K and trms might be wrong / are off
%      .warn_maxiter   warning: maximum number of iterations reached
%      .warn_maxrays   warning: maximum number of paths reached
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
% - sampling of spaced-frequency correlation function (does not work so far)
%   
% *******************************************************************************************************


function [delay_s, gain, statistics] = channel_small(settings)
version = 'beta 1.0';


% *******************************************************************************************************
% version system

% just return version number
if nargin == 0
   delay_s = version;
   return
end

% call version system (logs version numbers)
version_system();


% *******************************************************************************************************
% internal settings

% do not increase tmax any more if tmax*gamma > internalsettings.tg_max to prevent Inf in equations
internalsettings.tg_max = log(1/eps)-6; % exp(internalsettings.tg_max) in equations!


% *******************************************************************************************************
% input parameter checks

% check contents of settings
%     prepare required data
expected.name = 'settings';
expected.reqfields = {'on','det','seed','ensembles','maxrays', 'k','trms','eps_k','eps_trms','bw','fres', 'fs'};
%     check
errortext = contentcheck(settings, expected);
%     output
if ~isempty(errortext)
   err('Incomplete settings\n%s', errortext);
end

% initialize warning switches
statistics.warn_res     = false; % there might be problems with the resulting frequency resolution
statistics.warn_bw      = false; % there might be problems with the resulting bandwidth
statistics.warn_ktrms   = false; % K and trms might be wrong / are off
statistics.warn_maxiter = false; % maximum number of iterations reached
statistics.warn_maxrays = false; % maximum number of paths reached

% return if switched off (this should be done outside this function)
if ~settings.on
   warn('This function has been called "switched off". Is this a mistake?');
   delay_s = 0;
   gain    = 1;
   statistics.av_k    = Inf;
   statistics.av_trms =   0;
   return
end


% *******************************************************************************************************
% create average Power-Delay-Profile [1] with linear spacing meeting settings as good as possible
% ... this is like sampling the spaced-frequency-correlation-function with subsequent upsampling without
%     filtering => there will be mirrors outside statistics.bw

% K-factor: [dB] -> linear
settings.k_lin = 10^(settings.k/10);

% parameters of PDP (overall power of 0 dB ... P0=1)
los   = settings.k_lin/(settings.k_lin+1); % power of LOS component
gamma = 1/settings.trms * sqrt(2*settings.k_lin+1)/(settings.k_lin+1); % decay constant of exp. distribution
nlos  = 1/(settings.k_lin+1) * gamma; % power density NLOS

% determine number of samples and maximum delay to meet setup
%     maximum delay
%       .) K-factor only (unfortunately this cannot be done analytically for RMS delay spread)
%       .) minimum frequency resolution
tmax = max(1/settings.fres, -log(settings.eps_k/(settings.eps_k+1))/gamma);
tmax = ceil(tmax * settings.fs) / settings.fs; % round to sampling interval
if gamma*tmax > internalsettings.tg_max % truncate if necessary
   statistics.warn_res   = true;
   statistics.warn_ktrms = true;
   tmax = internalsettings.tg_max/gamma;
end
%     number of taps to meet given bandwidth
settings.n = ceil(2*tmax*settings.bw);
if settings.n > settings.maxrays
   statistics.warn_maxrays = true;
   settings.n = settings.maxrays;
end
%     force number of paths?
if isfield(settings, 'nrays_f') && ~isnan(settings.nrays_f) && settings.nrays_f >= 0
   critwarn('Forced number of paths. Note that this has priority over most other settings.');
   settings.n = settings.nrays_f;
   settings.maxiter = 1;
end

% iteratively generate average PDP (try to get rms delay spread right)
for iter = 1 : settings.maxiter
   % create average PDP
   %     placement of integration bounds
   intb = linspace(0, tmax, settings.n + 1)';
   %     placement of delays at centers of gravity (hopefully the rounding won't be much of a problem)
   delay_s = [0; round( ( exp(intb(1:end-1)*gamma) .* (1+intb(2:end)*gamma) - exp(intb(2:end)*gamma) .* (1+intb(1:end-1)*gamma) ) ./...
      (gamma * ( exp(intb(1:end-1)*gamma) - exp(intb(2:end)*gamma) ) ) * settings.fs )];
   %     generate gains
   gain = sqrt([los; -nlos/gamma * ( exp(-gamma*intb(2:end))-exp(-gamma*intb(1:end-1)) )]);

   % check for identical entries in delay_s (too many rays for the given timings)
   if any(diff(delay_s)==0)
      % output a warning after the iteration is complete
      statistics.warn_res = true;
      % get a list with identical delays
      id_del = delay_s(diff(delay_s)==0);
      id_del(diff(id_del)==0) = [];
      % browse through list and combine gains for identical entries
      for i = 1 : length(id_del)
         ind = find(delay_s(:)==id_del(i));
         gain(ind(1)) = sqrt(sum(gain(ind).^2)); % geometric add.
         delay_s(ind(2:end)) = [];
         gain(ind(2:end))    = [];
      end
   end
   
   % calculate statistics
   statistics.av_k    = 10*log10(gain(1)^2/sum(gain(2:end).^2)); % Ricean K-Factor in dB
   statistics.av_trms = sqrt(var(delay_s/settings.fs, gain.^2)); % RMS delay spread
         
   % check RMS delay spread
   if abs(statistics.av_trms - settings.trms) / settings.trms > settings.eps_trms
      % we have already reached maximum number of rays => increased # of paths not possible
      if statistics.warn_maxrays
         break;
      end
      % is this the last round (i.e., do we have no more chance to adjust trms)?
      if iter == settings.maxiter
         statistics.warn_maxiter = true;
      end
      % no: trms asymptotically correct for settings.n>> tmax>> 
      % => increase resolution (number of paths) ...
      settings.n = ceil(settings.n *...
         (1+sqrt((abs(statistics.av_trms-settings.trms)/settings.trms-settings.eps_trms)/settings.eps_trms)) );
      % ...and tmax if we have'nt increased it to more than internalsettings.tg_max time constants yet
      tmax_test = tmax * (1+sqrt((abs(statistics.av_trms-settings.trms)/settings.trms-settings.eps_trms)/settings.eps_trms));
      if gamma*tmax_test < internalsettings.tg_max
         tmax = tmax_test;
      else
         statistics.warn_ktrms = true;
      end
      
      % truncate if necessary
      if settings.n > settings.maxrays
         statistics.warn_maxrays = true;
         settings.n = settings.maxrays;
      end
   else
      break;
   end
end
%     record for statistics...
statistics.iter = iter;
statistics.n = length(delay_s);
statistics.fres = 1 / tmax;
statistics.bw = statistics.n / (2*tmax);

% output warnings
if statistics.warn_res || statistics.warn_bw || statistics.warn_ktrms ||...
      statistics.warn_maxiter || statistics.warn_maxrays
   critwarn('Unable to meet settings. Please check statistics for details.');
   % most important details only if messages are enabled
   text = '';
   if statistics.warn_res; text=strcat(text, ', freq.res'); end
   if statistics.warn_bw; text=strcat(text, ', BW'); end
   if statistics.warn_ktrms; text=strcat(text, ', K/trms'); end
   if statistics.warn_maxiter; text=strcat(text, ', maxiter'); end
   if statistics.warn_maxrays; text=strcat(text, ', maxrays'); end
   message('The following problems occured: %s', text(3:end));
end


% *******************************************************************************************************
% randomize PDP (if not "deterministic")
% ... replace NLOS gains by zero-mean Gaussian RV with current variance
% ... also randomize delay_s to create random phases for smallscale systems

if ~settings.det
   % given seed
   if ~isnan(settings.seed)
      warn('Setting RandStream (mt19937ar) seed to %s (will be reset to prior state before this fcn returns).',...
         strtrim(sprintf('%15.0f', settings.seed)));
      settings.oldDefaultStream = RandStream.getDefaultStream;
      RandStream.setDefaultStream(RandStream('mt19937ar','seed',settings.seed));
   end
   % more than one ensemble?
   if isfield(settings, 'ensembles') && settings.ensembles > 1
      gain = [gain(1)*ones(1, settings.ensembles); ...
         randn(length(gain)-1, settings.ensembles) .* repmat(gain(2:end), 1, settings.ensembles)];
      delay_s = [delay_s(1)*ones(1, settings.ensembles); ...
         repmat(delay_s(2:end),1,settings.ensembles) + round((rand(length(delay_s)-1,settings.ensembles)-0.5)*0.9*min(diff(delay_s)))];
   else
      gain(2:end)    = randn(length(gain)-1, 1) .* gain(2:end);
      delay_s(2:end) = delay_s(2:end) + round((rand(length(delay_s)-1,1)-0.5)*0.9*min(diff(delay_s)));
   end
   % given seed: make sure everything from here is "random" again
   if ~isnan(settings.seed)
      RandStream.setDefaultStream(settings.oldDefaultStream);
   end
end

% make sure delay_s is strictly monotonically rising
if any(any(diff(delay_s)<=0))
   err('Delay vector is not strictly monotonically rising. Check PDP generation.');
end
