% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% MFCW main script - 3D gate simulation with MFCW ranging 
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Suitable for Batch-Processing *****
% This module is intended to be used as script (started manually and without the possibility of changing
% settings and/or the behavior). Nontheless it offers a minimalistic interface which enables automated 
% external batch processing. Please refer to the m-file itself for interface definition and behavior,
% as neither interface, nor behavior are fixed.
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
% - decrease carrier length in field probe mode after leading/trailing zeros have been removed from est.
%
% *******************************************************************************************************

function mfcw_gate3d(N_sim)


% *******************************************************************************************************
% initialization
if nargin == 0
   clear; clc; close all; pause(0.01);
end

% paths, etc
%     path to globalinit.m
path = fullfile(fileparts(mfilename('fullpath')), '..', '..');
%     Matlab does not resolve symbolic links; if we're running on Unix, let the system resolve them
if isunix
   [dummy, path] = system(sprintf('readlink -f "%s"', path));
end
%     add this path and call globalinit script
restoredefaultpath; addpath(path); clear('dummy', 'path');
globalsettings_copy = globalinit('verbose');

% prevent full execution by version_system
if get_stacklevel > 0
   return
end

% "switch to here"
cd(fileparts(mfilename('fullpath')));


% *******************************************************************************************************
% setup simulator

% simulation environment
%     general settings
if nargin == 0
   N_sim = 0;
end
settings.suffix        = sprintf('mfcw_gate3d__TEST-%02i', N_sim); % filename suffix for all output files (may be reset below)
settings.renew_kticket =   true; % renew Kerberos ticket and AFS token?
settings.renew_kticket =  false;
settings.err_rep       = ~globalsettings_copy.core.debug; % try to repeat loop passes in case of errors (-> link to debug mode)
%     folders
settings.folders.root = fullfile(globalsettings_copy.path.master, 'simulator');
settings.folders.data = '_data/mfcw_gate3d';
settings.folders.logs = '_logs/mfcw_gate3d';
%     hostname and process id
settings.hostname = globalsettings_copy.misc.hostname;
settings.pid      = globalsettings_copy.misc.pid;

% gate scenario
settings.scenario = 'random';
settings.scenario = 'curve';

% specials
settings.specials.probemode      =   false; % set all tags to "field probe mode", i.e., just record the field strength
settings.specials.probemode_clen =   1e-4; % s carrier length(s) in "field probe mode"

% ETA eMails
settings.emails.eta_sendmail = true; % sending of an eMail containing the ETA of a simulation
settings.emails.eta_atloop   =   10; % send ETA eMail in this loop (index)
settings.emails.eta_mintime  =   24; % hrs minimum overall simulation time to send ETA eMail

% constants
%     sampling frequency (set to multiple of fi times nfft for best resolution)
settings.fs = 3e9; % Hz 
%     speed of light
settings.c = 299792458; % m/s

% channels (cf. signal model!)
%     switches
settings.channel_global.noise_on = 0; % noise on ? 
settings.channel_global.refl_on  = 0; % reflective surfaces on ?
settings.channel_global.small_on = 0; % smallscale channels on ?
%     noise
settings.channel_global.n0       = [-47, 3000]; % single-sided noise density [dBm, per ? Hz]
%     reflective surface: global
settings.channel_global.pol_dim  = 3; % linear polarization along z-axis for all devices
%     reflective surface: concrete floor in xy-plane
settings.channel_global.reflections.floor.dim   =   3; % normal to z-axis ([x,y,z])
settings.channel_global.reflections.floor.shift =   0; % at z=0
settings.channel_global.reflections.floor.n2    = 2.5; % refractive index
%     smallscale
settings.channel_global.v_dist = linspace(        0,       10, 1000); % m (start at 0 m!)
settings.channel_global.v_k    = logspace(log10(30), log10(6), 1000); % dB
settings.channel_global.v_trms = linspace(     1e-9,    20e-9, 1000); % s


% *******************************************************************************************************
% readers and tags

% readerpool ({reader1, reader2, reader3, ...})
settings.readerpool.n       = 4; % number of readers
settings.readerpool.fc      = {   915e6,    915e6,    915e6,    915e6}; % Hz carrier frequency
settings.readerpool.ant     = {... % antenna type (characteristic filename); isotropic antenna if empty
   'channelchar_directivity_intermec-ia39b', 'channelchar_directivity_intermec-ia39b',...
   'channelchar_directivity_intermec-ia39b', 'channelchar_directivity_intermec-ia39b'};
settings.readerpool.ant_rot = {   [0,0],  [180,0],    [0,0],  [180,0]}; % antenna rotation [azimuth, elevation] in degree ([0,0]: maximum in positive x-axis)
settings.readerpool.ptx     = {    3.28,     3.28,     3.28,     3.28}; % W EIRP transmit power
settings.readerpool.t0      = {    0e-6,     0e-6,     0e-6,     0e-6}; % s time delay
settings.readerpool.id_ch   = {    true,     true,     true,     true}; % identical smallscale channels to this reader? (does the reader use a single TX/RX antenna?) 
settings.readerpool.frs     = {    65e6,     65e6,     65e6,     65e6}; % Hz reader sampling frequency
settings.readerpool.quant   = {     NaN,      NaN,      NaN,      NaN}; % bits quantization (set to NaN to deactivate)
settings.readerpool.pos     = {[-1.5,0,1.5],[1.5,0,1.5],[-1.5,0,0.5],[1.5,0,0.5]}; % m [x,y,z] position of readers 

% MFCW ranging (part of readerpool)
settings.readerpool.mfcw.nc   = {1, 1, 1, 1}; % number of secondary carriers
%     secondary carrier frequencies in Hz
settings.readerpool.mfcw.fi   = {[ 5e6], [ 5e6], [ 5e6], [ 5e6]};
%     secondary carrier variances (main carrier has variance 1)
settings.readerpool.mfcw.vari = {[1e-3], [1e-3], [1e-3], [1e-3]}; 

% tagpool ({tag1, tag2, tag3, ...})
settings.tagpool.n         =            1; % number of tags
settings.tagpool.t0        =       {0e-6}; % s time delay
settings.tagpool.rn16      =         {''}; % zero-length RN16 fake (faster ranging, less RAM)
settings.tagpool.ant       = {'channelchar_directivity_l2-dipole'}; % antenna type (characteristic filename); isotropic antenna if empty
settings.tagpool.ant_rot   =      {[0,0]}; % antenna rotation [azimuth, elevation] in degree ([0,0]: maximum in positive x-axis)
settings.tagpool.pwrchar   = {'Vdda_AVG'}; % power supply characteristic {'Vdda_V2A', 'Vdda_V2B', 'Vdda_V2C', 'Vdda_AVG'}
settings.tagpool.modcharid =    {'b-p2a'}; % ID for tag modulator characteristic
settings.tagpool.cat       =    {400e-15}; % F assembly/application capacity (tag modulator characteristic)
settings.tagpool.rat       =        {  0}; % percent shift for assembly/application resistance (tag modulator characteristic)
settings.tagpool.pos       =   {nan(3,1)}; % m [x,y,z] position of tags for initialization (set below)
%     set tag positions 2 meter from Reader1 in direction of main lobe for initialization
[x,y,z] = sph2cart(settings.readerpool.ant_rot{1}(1), settings.readerpool.ant_rot{1}(2), 2);
settings.tagpool.pos = cell(settings.tagpool.n, 1);
settings.tagpool.pos = cellfun(@(pos) settings.readerpool.pos{1}+[x,y,z], settings.tagpool.pos, 'UniformOutput', false);
clear('x','y','z');

% tag movement (positions will be set randomly below)
settings.loop.n    = 100; % number of random tag postions to simulate
settings.loop.bnds = [-5,5; -5,5; 0,2]; % [xmin,xmax; ymin,ymax; zmin,zmax] bounds for tag placement


% *******************************************************************************************************
% predefined scanarios (single tag)

settings.scenario = 'curve';
  
switch lower(settings.scenario)
   case 'random'; % create random tag positions, drop whenever position is too close to a reader
      settings.loop.pos_t{1} = zeros(settings.loop.n, 3); % [x,y,z] position of tag in meters
      i = 0;
      while true
         newpos = [settings.loop.bnds(1,1) + diff(settings.loop.bnds(1,1:2))*rand,...
            settings.loop.bnds(2,1) + diff(settings.loop.bnds(2,1:2))*rand,...
            settings.loop.bnds(3,1) + diff(settings.loop.bnds(3,1:2))*rand];
         if min(cellfun(@(x) norm(x-newpos), settings.readerpool.pos)) > 0.3 % accept point
            i = i + 1;
            settings.loop.pos_t{1}(i,:) = newpos;
            if i == settings.loop.n; break; end
         else
            continue;
         end
      end
   case 'curve' % some curve in space [x,y,z]
      settings.loop.pos_t{1} = [sin(linspace(-pi/3,pi,settings.loop.n))',...
         linspace(-2,2,settings.loop.n)', 1 + cos(linspace(0,pi,settings.loop.n))'];
   otherwise
      disp('Unsupported scenario...'); return
end
   
% plot setup
%     tags
tagpos = cell2mat(settings.loop.pos_t);
%     direction of readers (not vectorized for simplicity)
for i = 1 : settings.readerpool.n
   rdrant = load(settings.readerpool.ant{i});
   if rdrant.full3d
      [az, el] = meshgrid(rdrant.az(1:3:end)+settings.readerpool.ant_rot{i}(1), rdrant.el(1:3:end)+settings.readerpool.ant_rot{i}(2));
      [rdrpos_dir.x{i}, rdrpos_dir.y{i}, rdrpos_dir.z{i}] =...
         sph2cart(az*pi/180, pi/2-el*pi/180, rdrant.gain(1:3:end, 1:3:end)');
   else
      rdrpos_dir.x{i} = (rdrant.el_gain(1:3:end) .* sin((rdrant.el(1:3:end)+settings.readerpool.ant_rot{i}(2))*pi/180)) *...
         (rdrant.az_gain(1:3:end) .* cos((rdrant.az(1:3:end)+settings.readerpool.ant_rot{i}(1))*pi/180))';
      rdrpos_dir.y{i} = (rdrant.el_gain(1:3:end) .* sin((rdrant.el(1:3:end)+settings.readerpool.ant_rot{i}(2))*pi/180)) *...
         (rdrant.az_gain(1:3:end) .* sin((rdrant.az(1:3:end)+settings.readerpool.ant_rot{i}(1))*pi/180))';
      rdrpos_dir.z{i} = (rdrant.el_gain(1:3:end) .* cos((rdrant.el(1:3:end)+settings.readerpool.ant_rot{i}(2))*pi/180)) * (rdrant.az_gain(1:3:end) )';
   end
   rdrpos_dir.c{i} = sqrt( rdrpos_dir.x{i}.^2 + rdrpos_dir.y{i}.^2 + rdrpos_dir.z{i}.^2 );
   rdrpos_dir.x{i} = 1/2*rdrpos_dir.x{i} + settings.readerpool.pos{i}(1);
   rdrpos_dir.y{i} = 1/2*rdrpos_dir.y{i} + settings.readerpool.pos{i}(2);
   rdrpos_dir.z{i} = 1/2*rdrpos_dir.z{i} + settings.readerpool.pos{i}(3);
end
%     plot
figure; hold on;
for i = 1 : settings.readerpool.n
   mesh(rdrpos_dir.x{i}, rdrpos_dir.y{i}, rdrpos_dir.z{i}, rdrpos_dir.c{i});
end
plot3(tagpos(:, 1:3:end-2), tagpos(:, 2:3:end-1), tagpos(:, 3:3:end), 'b.');
hold off; grid on; view([40,40]); axis equal;
view([30,50]); setlabels('GATE SETUP WITH TAG AND READER POSITIONS', 'x [m]', 'y [m]', 'z [m]');
%     maximum distances
% cellfun(@(x) max(sqrt(sum((repmat(x,settings.loop.n,1)-settings.loop.pos_t{1}).^2, 2))), settings.readerpool.pos)
%     wait
disp('Press any key to continue...'); pause; close(gcf); pause(0.01);



% *******************************************************************************************************
% simulate

% create directories (return values to suppress warning if directory already exists)
[dummy.stat,dummy.msg,dummy.msgid] = mkdir(settings.folders.root, settings.folders.data);
[dummy.stat,dummy.msg,dummy.msgid] = mkdir(settings.folders.root, settings.folders.logs);

% inform the user: simulation has started
send_email(sprintf('Started %s (%i readers, %i tags, %i positions)', settings.suffix,...
   settings.readerpool.n, settings.tagpool.n, settings.loop.n), '');

% run simulator main file (will save ouput in files)
sim_mfcw_gate(settings);

% inform the user: simulation has been completed
send_email(sprintf('Completed %s (%i readers, %i tags, %i positions)', settings.suffix,...
   settings.readerpool.n, settings.tagpool.n, settings.loop.n), '');

% quit Matlab if this module was started in batch processing mode
if nargin > 0
   quit;
end
