% *******************************************************************************************************
% Position Aware RFID Systems: The PARIS Simulation Framework
% ***********************************************************
% MFCW main script - statistics of distance error for different distances / channels
%
%
%
% ***** Copyright / License / Authors *****
% Copyright 2007, 2008, 2009, 2010 Daniel Arnitz
%   Signal Processing and Speech Communication Laboratory, Graz University of Technology, Austria
%   NXP Semiconductors Austria GmbH Styria, Gratkorn, Austria
%
% This file is part of the PARIS Simulation Framework.
%
% The PARIS Simulation Framework is free software: you can redistribute it and/or modify it under the
% terms of the GNU General Public License as published by the Free Software Foundation, either
% version 3 of the License, or (at your option) any later version.
%
% The PARIS Simulation Framework is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along with the PARIS Simulation 
% Framework. If not, see <http://www.gnu.org/licenses/>.
%
%
% ***** Changelog *****
% REVISION   DATE         USER        DESCRIPTION (! bugfixes, + addons, - removals, ~ otherwise)
% beta 1.0   2010-03-30   arnitz      ~ initial release
%
%
% ***** Todo *****
%
% *******************************************************************************************************


function mfcw_statistics(v_k, v_trms)


% *******************************************************************************************************
% initialization
if nargin == 0
   clear; clc; close all; pause(0.01);
end

% paths, etc
%     path to globalinit.m
path = fullfile(fileparts(mfilename('fullpath')), '..');
%     Matlab does not resolve symbolic links; if we're running on Unix, let the system resolve them
if isunix
   [dummy, path] = system(sprintf('readlink -f "%s"', path));
end
%     add this path and call globalinit script
addpath(path); clear('dummy', 'path');
globalsettings_copy = globalinit('uc');

% prevent full execution by version_system
if get_stacklevel > 0
   return
end

% "switch to here"
cd(fileparts(mfilename('fullpath')));


% *******************************************************************************************************
% setup simulator

% simulation environment
%     general settings
settings.file_prefix   = 'mfcw_stat_test'; % filename suffix for all output files (may be reset below) 
settings.renew_kticket =        true; % renew Kerberos ticket and AFS token? 
settings.err_rep       = ~globalsettings_copy.core.debug; % try to repeat loop passes in case of errors (-> link to debug mode)
%     folders
settings.folders.root = fullfile(globalsettings_copy.path.master, 'simulator');
settings.folders.data = '_data/mfcw_statistics';
settings.folders.logs = '_logs/mfcw_statistics';
%     hostname and process id
settings.hostname = globalsettings_copy.misc.hostname;
settings.pid      = globalsettings_copy.misc.pid;

% ETA eMails
settings.emails.eta_sendmail = true; % sending of an eMail containing the ETA of a simulation
settings.emails.eta_atloop   =   10; % send ETA eMail in this loop (index)
settings.emails.eta_mintime  =   24; % hrs minimum overall simulation time to send ETA eMail

% constants
%     sampling frequency (set to multiple of fi times nfft for best resolution)
settings.fs = 3e9; % Hz 
%     speed of light
settings.c = 299792458; % m/s

% channels
%     switches
settings.channel_global.noise_on = 0; % activate noise ?
settings.channel_global.refl_on  = 1; % reflective surfaces on ? (keep FALSE)
settings.channel_global.small_on = 1; % activate smallscale channels ?
%     noise / smallscale
settings.channel_global.n0     = [-47, 3000]; % single-sided noise density [dBm, per ? Hz]
settings.channel_global.v_dist = linspace(0, 10, 10)'; % m (start at 0 m!)
%     reflective surface: global
settings.channel_global.pol_dim  = 2; % linear polarization along y-axis for all devices
%     reflective surface: concrete floor in y
settings.channel_global.reflections.floor.dim   =   1; % normal to x-axis ([x,y,z])
settings.channel_global.reflections.floor.shift =  10; % at x=10
settings.channel_global.reflections.floor.n2    = 2.5; % refractive index

% loop inside sim_mfcw_ranging
settings.loop.n     =                             1; % number of points in loop
settings.loop.pos_t = {ones(settings.loop.n,1) * 3}; % distance in m of tag(s)
%     round position to sampling interval
settings.loop.pos_t{1} = round(settings.loop.pos_t{1}/settings.c*settings.fs)*settings.c/settings.fs;


% *******************************************************************************************************
% readers and tags

% readerpool ({reader1, reader2, reader3, ...})
settings.readerpool.n       =       1; % number of readers  -  ONE READER ONLY!
settings.readerpool.fc      = {910e6}; % Hz carrier frequency (set to integer fraction of fs)
settings.readerpool.ant     = {'channelchar_directivity_intermec-ia39b'}; % antenna type (characteristic filename); isotropic antenna if empty
settings.readerpool.ant_rot = {[0,0]}; % antenna rotation [azimuth, elevation] in degree ([0,0]: maximum in positive x-axis)
settings.readerpool.ptx     =   {3.2}; % W EIRP transmit power
settings.readerpool.t0      =  {1e-6}; % s time delay of modulation
settings.readerpool.id_ch   =     {1}; % identical smallscale channels to this reader? (does the reader use a single TX/RX antenna?) 
settings.readerpool.frs     =  {50e6}; % Hz reader sampling frequency
settings.readerpool.quant   =   {NaN}; % bits quantization (set to NaN to deactivate)
settings.readerpool.pos     =   {[0]}; % m [x] position of readers 

% MFCW ranging (part of readerpool)
settings.readerpool.mfcw.nc   = {1}; % number of secondary carriers -  ONE RANGING CARRIER ONLY!
%     secondary carrier frequencies in Hz
settings.readerpool.mfcw.fi   = {[10]*1e6};
%     secondary carrier variances (main carrier has variance 1) 
settings.readerpool.mfcw.vari = {[1e-2]}; 

% tagpool ({tag1, tag2, tag3, ...})
settings.tagpool.n         =            1; % number of tags  -  ONE TAG ONLY!
settings.tagpool.t0        =       {0e-6}; % s time delay
settings.tagpool.rn16      =         {''}; % zero-length RN16 fake (faster ranging, less RAM)
settings.tagpool.ant       = {'channelchar_directivity_l2-dipole'}; % antenna type (characteristic filename); isotropic antenna if empty
settings.tagpool.ant_rot   =      {[0,0]}; % antenna rotation [azimuth, elevation] in degree ([0,0]: maximum in positive x-axis)
settings.tagpool.pwrchar   = {'Vdda_V2B'}; % power supply characteristic {'Vdda_V2A', 'Vdda_V2B', 'Vdda_V2C', 'Vdda_AVG'}
settings.tagpool.modcharid =    {'c-p2a'}; % ID for tag modulator characteristic
settings.tagpool.cat       =    {200e-15}; % F assembly/application capacity (tag modulator characteristic)
settings.tagpool.rat       =          {0}; % percent shift for assembly/application resistance (tag modulator characteristic)
settings.tagpool.pos       = {[settings.loop.pos_t{1}(1)]}; % m [x] position of tags


% *******************************************************************************************************
% prepare

% create directories (return values to suppress warning if directory already exists)
[dummy.stat,dummy.msg,dummy.msgid] = mkdir(settings.folders.root, settings.folders.data);
[dummy.stat,dummy.msg,dummy.msgid] = mkdir(settings.folders.root, settings.folders.logs);


% *******************************************************************************************************
% simulate

if nargin == 0
   v_k    = 30:-6:0; % dB
   v_trms = [2, 5, 10, 20]; % ns
end

% loop
for k = 1 : length(v_k)
   for t = 1 : length(v_trms)
         % modify setup, round distance to sampling resolution (remove this systematic error)
         if settings.readerpool.id_ch{1}
            settings.suffix = sprintf('%s_id_%02.0fMHz_K%02.0f_trms%02.0f',...
               settings.file_prefix, settings.readerpool.mfcw.fi{1}/1e6, v_k(k), v_trms(t));
         else
            settings.suffix = sprintf('%s_nid_%02.0fMHz_K%02.0f_trms%02.0f',...
               settings.file_prefix, settings.readerpool.mfcw.fi{1}/1e6, v_k(k), v_trms(t));
         end
         settings.channel_global.v_k    = ones(size(settings.channel_global.v_dist)) * v_k(k); % dB
         settings.channel_global.v_trms = ones(size(settings.channel_global.v_dist)) * v_trms(t) * 1e-9; % s         
         % inform the user: a new simulation has started
         send_email(sprintf('Started %s (%.0f of %.0f; %.0f iterations)', settings.suffix,...
            (k-1)*length(v_trms)+t, length(v_k)*length(v_trms), settings.loop.n), '');
         % run simulator main file (will save ouput in files)
         sim_mfcw_ranging(settings);
         % inform the user: a new simulation has been completed and wait a little
         send_email(sprintf('Completed %s (%.0f of %.0f)', settings.suffix,...
            (k-1)*length(v_trms)+t, length(v_k)*length(v_trms)), '');
         pause(10); % wait until mail is delivered before sending the next one at beginning of loop
   end
end

% quit Matlab if this module was started in batch processing mode
if nargin > 0
   quit;
end
